/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        Graphic LCD.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
* 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES: :
*
* For Use With T693C 128x64 Graphic LCD    T693C 128x64  LCD
* Use With SPI GPIO For Data Lines (See SPI GPIO.h)   SPI GPIO    (. SPI GPIO.h),
*   SPI GPIO    (. SPI GPIO.h),
**********************************************************************/
//INCLUDES 
	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include <dsp.h>
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"
	#include "SPI GPIO.h"
	#include "LCR METER FUNCS.h"


/********************************************************************/
//   ()

	#define	UPFREQ					24000000		//For Use With Delay Function      	

	//   8  ( 8bit   16bits), 
	#define	LOBYTE(arg)				(arg & 0xFF)
	#define	HIBYTE(arg)				((char)(arg >> 8))

	//   LCD
	#define	ReadWrite				LATEbits.LATE1	//Read Signal (Write provided by NPN transistor),    N-P-N), 
	#define	CommandData				LATEbits.LATE3	//Command \ Data Line  \  
	#define	ChipEnable				LATEbits.LATE4	//Chip Enable Line   

	//  
	#define	OR_MODE					0x80
	#define	XOR_MODE				0x81
	#define	AND_MODE				0x82

	#define GRAPH_HOME_SET			0x0000
	#define GRAPH_AREA_SET			0x16
	#define	GRAPH_LINES				0x40

	#define	LOWER_FRAME				0x03F4

	#define	SET						0xFF
	#define	CLEAR					0x00

	#define	TEXT_HOME_SET			0x0590
	#define TEXT_AREA_SET			0x16
	#define TEXT_LINES				0x08

	#define	SCREEN_FRAME_LINES		0X05
	#define	SCREEN_UPPER_HOME		0X00
	#define SCREEN_LOWER_HOME   	0X512

	#define	CIRCUIT_ELEMENT_WIDTH 	0X04
	#define CIRCUIT_ELEMENT_LINES	0X09

	#define SYMBOL_NODE_WIDTH 		0x02
	#define SYMBOL_NODE_LINES 		0X05

	#define NODE_LEFT_HOME			0X176
	#define NODE_RIGHT_HOME			0X180

	#define SERIES_LEFT_HOME 		0X14C
	#define SERIES_RIGHT_HOME 		0X150
	#define PARALELL_TOP_HOME		0X72
	#define PARALELL_BOTTOM_HOME 	0X22A

	#define	SYMBOL_PARALEL_WIDTH 	0X02
	#define	SYMBOL_PARALEL_LINES 	0X15

	#define SYMBOL_PARALEL_LEFT_HOME 0XC8
	#define SYMBOL_PARALEL_RIGHT_HOME 0XCE
	#define	CIRCUIT_AREA_HOME 		0X45

	#define	MODE_HOME				0x70
	#define MODE_CHAR				0x08
	#define FREQ_HOME				0x86
	#define FREQ_CHAR				0x07

	#define DATA_NUM_HOME 			0X21

	#define	SYMBOL_TEXT_WIDTH 		0x01
	#define	SYMBOL_TEXT_LINES 		0x08

	#define	SYMBOL_THETA_HOME 		0x42C
	#define	SYMBOL_DEG_HOME	  		0x433
	#define	SYMBOL_OHM_HOME   		0xC3

	// 

	#define		CAP					0x00
	#define		IND					0x01	

	#define		F10KHz				0x03	
	#define		F1KHz				0x02
	#define		F100Hz				0x01
	#define		AUTOHz				0x00

	#define		CALIBRATION_MODE 	0x04
	#define		TESTING_MODE 		0x03
	#define		PARALLEL_MODE		0x02
	#define		SERIES_MODE 		0x01
	#define		AUTO_MODE			0x00

	#define		DASH_SYM			0x00
	#define		RESISTOR_SYM 		0x01
	#define		CAPACITOR_SYM 		0x02
	#define		INDUCTOR_SYM		0x03

	#define		OUTPUT_HOME			0x20


/*****************************************************************************/
// 


//  
const unsigned char SplashScreen[GRAPH_AREA_SET * GRAPH_LINES] = {
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,
0x3F,0x3F,0x3F,0x38,0x0F,0x3F,0x3F,0x3F,0x3F,0x38,0x00,0x03,
0x3F,0x3E,0x01,0x3F,0x3F,0x20,0x07,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x30,0x07,0x3F,0x3F,0x3F,0x3F,0x30,0x00,0x00,0x3F,0x3C,
0x00,0x3F,0x3E,0x00,0x00,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x30,
0x07,0x3F,0x3F,0x3F,0x3F,0x30,0x00,0x00,0x1F,0x3C,0x00,0x3F,
0x38,0x00,0x00,0x1F,0x3F,0x30,0x3F,0x3F,0x3F,0x30,0x07,0x3F,
0x3F,0x3F,0x3F,0x30,0x00,0x00,0x0F,0x3C,0x00,0x3F,0x30,0x00,
0x00,0x0F,0x3F,0x30,0x3F,0x3F,0x3F,0x30,0x07,0x3F,0x3F,0x3F,
0x3F,0x30,0x00,0x00,0x07,0x3C,0x00,0x3F,0x20,0x00,0x00,0x07,
0x3F,0x30,0x3F,0x3F,0x3F,0x30,0x07,0x3F,0x3F,0x3F,0x3F,0x30,
0x00,0x00,0x07,0x3C,0x00,0x3F,0x20,0x00,0x00,0x07,0x3F,0x30,
0x3F,0x3F,0x21,0x30,0x07,0x3F,0x00,0x1F,0x3F,0x30,0x03,0x20,
0x07,0x3C,0x00,0x3F,0x00,0x07,0x20,0x07,0x3F,0x30,0x3F,0x3E,
0x00,0x10,0x07,0x3C,0x00,0x03,0x3F,0x30,0x03,0x30,0x03,0x3C,
0x00,0x3F,0x00,0x0F,0x30,0x07,0x3F,0x30,0x3F,0x3C,0x00,0x00,
0x07,0x38,0x00,0x01,0x3F,0x30,0x03,0x30,0x03,0x3C,0x00,0x3F,
0x00,0x0F,0x38,0x0F,0x3F,0x30,0x3F,0x3C,0x00,0x00,0x07,0x38,
0x00,0x00,0x3F,0x30,0x03,0x30,0x03,0x3C,0x00,0x3E,0x00,0x1F,
0x3F,0x3F,0x3F,0x30,0x3F,0x38,0x00,0x00,0x07,0x30,0x07,0x20,
0x3F,0x30,0x03,0x20,0x03,0x3C,0x00,0x3E,0x00,0x1F,0x3F,0x3F,
0x3F,0x30,0x3F,0x38,0x03,0x20,0x07,0x30,0x07,0x3F,0x3F,0x30,
0x00,0x00,0x07,0x3C,0x00,0x3E,0x00,0x1F,0x3F,0x3F,0x3F,0x30,
0x3F,0x30,0x07,0x30,0x07,0x30,0x07,0x3F,0x3F,0x30,0x00,0x00,
0x07,0x3C,0x00,0x3E,0x00,0x1F,0x3F,0x3F,0x3F,0x30,0x3F,0x30,
0x07,0x30,0x07,0x38,0x00,0x0F,0x3F,0x30,0x00,0x00,0x0F,0x3C,
0x00,0x3E,0x00,0x1F,0x3F,0x3F,0x3F,0x30,0x3F,0x30,0x07,0x30,
0x07,0x38,0x00,0x00,0x3F,0x30,0x00,0x00,0x1F,0x3C,0x00,0x3E,
0x00,0x1F,0x3F,0x3F,0x3F,0x30,0x3F,0x30,0x07,0x30,0x07,0x3C,
0x00,0x00,0x1F,0x30,0x00,0x00,0x3F,0x3C,0x00,0x3F,0x00,0x1F,
0x38,0x0F,0x3F,0x30,0x3F,0x30,0x07,0x30,0x07,0x3F,0x00,0x00,
0x1F,0x30,0x00,0x03,0x3F,0x3C,0x00,0x3F,0x00,0x0F,0x30,0x07,
0x3F,0x30,0x3F,0x30,0x07,0x30,0x07,0x3F,0x3E,0x00,0x0F,0x30,
0x01,0x3F,0x3F,0x3C,0x00,0x3F,0x00,0x0F,0x30,0x07,0x3F,0x30,
0x3F,0x30,0x07,0x30,0x07,0x3F,0x3F,0x20,0x0F,0x30,0x03,0x3F,
0x3F,0x3C,0x00,0x3F,0x20,0x07,0x20,0x07,0x3F,0x30,0x3F,0x38,
0x03,0x20,0x07,0x38,0x1F,0x20,0x0F,0x30,0x03,0x3F,0x3F,0x3C,
0x00,0x3F,0x20,0x00,0x00,0x0F,0x3F,0x30,0x3F,0x38,0x00,0x00,
0x07,0x30,0x0F,0x20,0x1F,0x30,0x03,0x3F,0x3F,0x3C,0x00,0x3F,
0x30,0x00,0x00,0x0F,0x3F,0x30,0x3F,0x3C,0x00,0x00,0x07,0x30,
0x00,0x00,0x1F,0x30,0x03,0x3F,0x3F,0x3C,0x00,0x3F,0x38,0x00,
0x00,0x1F,0x3F,0x30,0x3F,0x3E,0x00,0x08,0x07,0x38,0x00,0x00,
0x3F,0x30,0x03,0x3F,0x3F,0x3C,0x00,0x3F,0x3C,0x00,0x00,0x3F,
0x3F,0x30,0x3F,0x3F,0x00,0x18,0x07,0x3C,0x00,0x01,0x3F,0x30,
0x03,0x3F,0x3F,0x3C,0x00,0x3F,0x3E,0x00,0x01,0x3F,0x3F,0x30,
0x3F,0x3F,0x31,0x3C,0x0F,0x3F,0x00,0x07,0x3F,0x38,0x07,0x3F,
0x3F,0x3E,0x01,0x3F,0x3F,0x20,0x07,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x30,0x33,0x3F,0x38,0x00,0x01,0x00,0x00,0x0F,0x38,0x00,
0x01,0x00,0x00,0x08,0x00,0x01,0x20,0x00,0x04,0x00,0x00,0x30,
0x33,0x3F,0x38,0x00,0x01,0x00,0x00,0x0F,0x38,0x00,0x01,0x00,
0x00,0x08,0x00,0x01,0x20,0x00,0x04,0x00,0x00,0x30,0x33,0x3F,
0x39,0x3F,0x3F,0x0F,0x3F,0x2F,0x39,0x39,0x39,0x0F,0x3F,0x3F,
0x39,0x3F,0x27,0x3F,0x3C,0x3F,0x3E,0x30,0x33,0x3F,0x39,0x3F,
0x3F,0x00,0x00,0x0F,0x39,0x39,0x39,0x00,0x00,0x0F,0x39,0x3F,
0x20,0x00,0x04,0x00,0x00,0x30,0x33,0x3F,0x39,0x3F,0x3F,0x00,
0x00,0x0F,0x39,0x39,0x39,0x00,0x00,0x0F,0x39,0x3F,0x20,0x00,
0x04,0x00,0x00,0x30,0x33,0x3F,0x39,0x3F,0x3F,0x0F,0x3E,0x1F,
0x39,0x39,0x39,0x0F,0x3F,0x3F,0x39,0x3F,0x27,0x3F,0x3C,0x3F,
0x39,0x30,0x30,0x00,0x08,0x00,0x01,0x0F,0x3F,0x0F,0x39,0x39,
0x39,0x00,0x00,0x0F,0x39,0x3F,0x20,0x00,0x04,0x3F,0x3C,0x30,
0x30,0x00,0x08,0x00,0x01,0x0F,0x3F,0x0F,0x39,0x39,0x39,0x00,
0x00,0x0F,0x39,0x3F,0x20,0x00,0x04,0x3F,0x3C,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
0x3F,0x3F,0x3F,0x30
} ;


//  
const unsigned int ScreenUpper[GRAPH_AREA_SET * SCREEN_FRAME_LINES] = {
0X03,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X31,
0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3C,0X00,0X0F,0X3F,
0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3C,0X3F,0X3F,
0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X00,0X1E,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X1E,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X07,0X20,0X18,0X00,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X06,0X00,0X00,0X00,0X00,0X00,0X00,
0X00,0X00,0X01,0X20,0X38,0X00,0X00,0X00,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X07,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,
0X01,0X30
} ;


//  
const unsigned char ScreenLower[GRAPH_AREA_SET * SCREEN_FRAME_LINES] = {
0X38,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X07,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X01,0X30,0X18,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X06,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0X01,0X20,0X1E,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X1E,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X07,0X20,0X0F,0X3F,0X3F,0X3F,0X3F,0X3F,
0X3F,0X3F,0X3F,0X3F,0X3F,0X3C,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,
0X3F,0X3F,0X3F,0X00,0X03,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,
0X3F,0X3F,0X3F,0X31,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,0X3F,
0X3C,0X00
} ;


//     (L)
const unsigned char SymbolInductor[CIRCUIT_ELEMENT_WIDTH * CIRCUIT_ELEMENT_LINES] = {
0X03,0X01,0X20,0X30,0X07,0X23,0X31,0X38,0X0C,0X36,0X1B,0X0C,
0X08,0X14,0X0A,0X04,0X38,0X14,0X0A,0X07,0X00,0X1C,0X0E,0X00,
0X00,0X1C,0X0E,0X00,0X00,0X08,0X04,0X00,0X00,0X00,0X00,0X00
} ;


//    (R)
const unsigned char SymbolResistor[CIRCUIT_ELEMENT_WIDTH * CIRCUIT_ELEMENT_LINES] = {
0X04,0X00,0X04,0X00,0X0E,0X00,0X0E,0X00,0X1B,0X00,0X1B,0X00,
0X31,0X20,0X31,0X20,0X20,0X31,0X20,0X31,0X00,0X1B,0X00,0X1B,
0X00,0X0E,0X00,0X0E,0X00,0X04,0X00,0X04,0X00,0X00,0X00,0X00
} ;


//    (C)
const unsigned char SymbolCapacitor[CIRCUIT_ELEMENT_WIDTH * CIRCUIT_ELEMENT_LINES] = {
0X00,0X0C,0X0C,0X00,0X00,0X0C,0X0C,0X00,0X00,0X0C,0X0C,0X00,
0X00,0X0C,0X0C,0X00,0X3F,0X3C,0X0F,0X3F,0X00,0X0C,0X0C,0X00,
0X00,0X0C,0X0C,0X00,0X00,0X0C,0X0C,0X00,0X00,0X0C,0X0C,0X00
} ;


//    (___)
const unsigned char SymbolDashLine[CIRCUIT_ELEMENT_WIDTH * CIRCUIT_ELEMENT_LINES] = {
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X26,0X19,0X26,0X19,0X00,0X00,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00
} ;


//   (  )
const unsigned char SymbolParalelLeft[SYMBOL_PARALEL_WIDTH * SYMBOL_PARALEL_LINES] = {
0X00,0X0F,0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,
0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,0X3F,0X38,0X00,0X08,
0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,0X00,0X08,
0X00,0X08,0X00,0X08,0X00,0X0F
} ;


//   (  ) 
const unsigned char SymbolParalelRight[SYMBOL_PARALEL_WIDTH * SYMBOL_PARALEL_LINES] = {
0X3C,0X00,0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,
0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,0X07,0X3F,0X04,0X00,
0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,0X04,0X00,
0X04,0X00,0X04,0X00,0X3C,0X00
} ;


//      ((.) 
const unsigned char SymbolNodeLeft[SYMBOL_NODE_WIDTH * SYMBOL_NODE_LINES] = {
0X30,0X30,0X31,0X38,0X33,0X3F,0X31,0X38,0X30,0X30
} ;


//     ((.)
const unsigned char SymbolNodeRight[SYMBOL_NODE_WIDTH * SYMBOL_NODE_LINES] = {
0X03,0X03,0X07,0X23,0X3F,0X33,0X07,0X23,0X03,0X03
} ;


//    (Ohm)
const unsigned char SymbolOhm[SYMBOL_TEXT_WIDTH * SYMBOL_TEXT_LINES] = {
0x00,0x0E,0x11,0x11,0x11,0x0A,0x1B,0x00
} ;


//   {}
const unsigned char SymbolDeg[SYMBOL_TEXT_WIDTH * SYMBOL_TEXT_LINES] = {
0x1C,0x14,0x1C,0x00,0x00,0x00,0x00,0x00
} ;


//    (Q)
const unsigned char SymbolTheta[SYMBOL_TEXT_WIDTH * SYMBOL_TEXT_LINES] = {
0x0E,0x11,0x11,0x1F,0x11,0x11,0x0E,0x00
} ;


// 10kHz ()
const unsigned char Freq10KHz[7] = {
' ','1','0','k','H','z',' '
} ;


// 1kHz ()
const unsigned char Freq1KHz[7] = {
' ',' ','1','k','H','z',' '
} ;


// 100Hz ()
const unsigned char Freq100Hz[7] = {
' ','1','0','0','H','z',' '
} ;


//  () 
const unsigned char FreqAUTOHz[7] = {
'A','U','T','O',' ','H','z'
} ;


//    ()
const unsigned char MsgParallel[8] = {
'P','A','R','A','L','L','E','L'
} ;


//    ()
const unsigned char MsgSeries[8] = {
' ','S','E','R','I','E','S',' '
} ;


//  ()  
const unsigned char MsgAuto[8] = {
' ',' ','A','U','T','O',' ',' '
} ;


//   ()
const unsigned char MsgRun_Cal[8] = {
'R','U','N',' ',' ','C','A','L'
} ;


//   () 
const unsigned char MsgTesting[8] = {
'T','E','S','T','I','N','G',' '
} ;


//  ()
const unsigned char MsgBlank[8] = {
' ',' ',' ',' ',' ',' ',' ',' '
} ;


/*****************************************************************************/
// 


extern struct {

	unsigned char MODE;
	unsigned char FREQ;

	unsigned char CAPINDTYPE;

	unsigned char DOMINANT_ELEMENT;
	unsigned char SECONDARY_ELEMENT;

	float	IMPEDANCE;
	float	RESISTANCE;
	float	COMPLEX_RES;
	float	CAP_IND;
	float	QUAL_DISAP;
	float 	PHASE_ANGLE;

} LCR_MEASUREMENTS;

unsigned char BUF_OUT[9];


/*****************************************************************************/


void WriteData (unsigned char Data) //    LCD
{

	// 
	int	wait;
	SET_SPI_GPIO_OUTPUT_VALUE(Data);
	CommandData=0; 					//  C\D 
	ReadWrite=0;					//   ( )
	wait=0;
	while	(wait < 4){wait++;}		//  
	ChipEnable=0;					//   
	wait=0;
	while	(wait < 4){wait++;}		// E   80ns 
	ChipEnable=1;					//   
	wait=0;
	while	(wait < 4){wait++;}		//  

}


////////////////////////////////////////


unsigned char ReadData () //    LCD
{

	// 
	int	wait;
	int Data;
	CommandData=0; 				//  C\D 
	ReadWrite=1;				//  ( )
	wait=0;
	while	(wait < 4){wait++;}	//  
	ChipEnable=0;				//   
	wait=0;
	while	(wait < 4){wait++;}	//  E   80ns
	Data = READ_SPI_GPIO_INPUT_VALUE();
	ChipEnable=1;				//    
	wait=0;
	while	(wait < 4){wait++;}	//   

return Data;
}


/////////////////////////////////////////


void WriteCommand (unsigned char Command) //    LCD
{
	// 
	int	wait;
	SET_SPI_GPIO_OUTPUT_VALUE(Command);
	CommandData=1; 	//  C\D 
	ReadWrite=0;	//    ( )
	wait=0;
	while	(wait < 4){wait++;}	//   
	ChipEnable=0;	//    
	wait=0;
	while	(wait < 4){wait++;}		//  E   80ns
	ChipEnable=1;	//    
	wait=0;
	while	(wait < 4){wait++;}	//   

}


//////////////////////////////////////////


unsigned char CheckStatus () //,  {}  LCD -   
{
	// 
	int	wait;
	int Data;
	CommandData=1; 				//High    
	ReadWrite=1;				//    ( )
	wait=0;
	while	(wait < 4){wait++;}	//   
	ChipEnable=0;				//    
	wait=0;
	while	(wait < 4){wait++;}	//  E   80ns
	Data = READ_SPI_GPIO_INPUT_VALUE();
	ChipEnable=1;				//    
	wait=0;
	while	(wait < 4){wait++;}	//   

return Data;
}


//////////////////////////////////////////


void InitLCD() //{}  LCD 
{
	// 
	int	Address;
	int MemCount;
	int Lines;	

	while ( (CheckStatus() & 0x03) != 0x03){}	// {} 
	WriteCommand(0x90); 	//     (OFF)

	while ( (CheckStatus() & 0x03) != 0x03){}	// {} 
	WriteCommand(OR_MODE); 	// 
	
	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS	
	WriteData(LOBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(HIBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x42); 	

	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(GRAPH_AREA_SET); 	
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(0x00); 
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x43); 	

	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(LOBYTE(TEXT_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(HIBYTE(TEXT_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x40); 	

	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(TEXT_AREA_SET); 	
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(0x00); 
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS  {} 
	WriteCommand(0x41); 

	//    
	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(LOBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(HIBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x24); 
	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS  {} 
	WriteCommand(0xB0); 						//START AUTOMODE  
	MemCount=(0x0590);

	//   
	while  (MemCount!=0)
	{	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
		WriteData(0xFF); 
		MemCount--;
	}	

	//  RAM 
	MemCount=(0x00B0);
	while  (MemCount!=0)
	{	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
		WriteData(0x00); 
		MemCount--;
	}	
	while ( (CheckStatus() & 0x08) != 0x08){}		//WAIT FOR STATUS BITS
	WriteCommand(0xB2); 							//END AUTOMODE  


	//   
	while ( (CheckStatus() & 0x03) != 0x03){}		//WAIT FOR STATUS BITS
	WriteCommand(0x9C); 							//GRAPHICS & TEXT ON    

	GenDelay(1000, UPFREQ); 						//Wait 1Sec  1Sec

	//   
	LoadSymbol (GRAPH_HOME_SET, SplashScreen, GRAPH_AREA_SET, GRAPH_LINES);

	GenDelay(2500, UPFREQ);							//Wait 2.5Sec 2.5Sec

	//SET GRAPHICS BLACK   
	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(LOBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(HIBYTE(GRAPH_HOME_SET));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x24); 
	// Auto Mode
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0xB0); 	//START AUTOMODE  
	MemCount=(0x0590);

	// 
	while  (MemCount!=0)
	{	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
		WriteData(0xFF); 
		MemCount--;
	}	
	while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
	WriteCommand(0xB2); 						//END AUTOMODE  

	GenDelay(1000, UPFREQ);						//Wait 1Sec  1Sec

	Address=0;
	//Set Home Address   
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(LOBYTE(SCREEN_UPPER_HOME));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteData(HIBYTE(SCREEN_UPPER_HOME));
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0x24); 

	//  
	while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
	WriteCommand(0xB0); 	//START AUTOMODE  

	MemCount=(GRAPH_AREA_SET * SCREEN_FRAME_LINES);

	while  (MemCount!=0)
	{	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
		WriteData(ScreenUpper[Address]); 
		MemCount--;
		Address++;
	}	

	Lines = 54;	
							
	while (Lines > 0)
	{
		WriteData(0x30); 
		//	
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 		
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 	
		//	
		WriteData(0x03); 
		//
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 		
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		WriteData(0x00); 
		//	
		WriteData(0x30); 
		Lines--;
	}

	Address = 0;
	MemCount=(GRAPH_AREA_SET * SCREEN_FRAME_LINES);

	while  (MemCount!=0)
	{	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS

		WriteData(ScreenLower[Address]); 
		MemCount--;
		Address++;
	}	

	while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
	WriteCommand(0xB2); 	//END AUTOMODE  

	//  
	LoadSymbol (NODE_LEFT_HOME, SymbolNodeLeft, SYMBOL_NODE_WIDTH, SYMBOL_NODE_LINES);

	//  
	LoadSymbol (NODE_RIGHT_HOME, SymbolNodeRight, SYMBOL_NODE_WIDTH, SYMBOL_NODE_LINES);

	//   
	LoadSymbol (SYMBOL_THETA_HOME, SymbolTheta, SYMBOL_TEXT_WIDTH, SYMBOL_TEXT_LINES);
	LoadSymbol (SYMBOL_DEG_HOME, SymbolDeg, SYMBOL_TEXT_WIDTH, SYMBOL_TEXT_LINES);
	LoadSymbol (SYMBOL_OHM_HOME, SymbolOhm, SYMBOL_TEXT_WIDTH, SYMBOL_TEXT_LINES);
	LoadSymbol ((SYMBOL_OHM_HOME + 0xB0), SymbolOhm, SYMBOL_TEXT_WIDTH, SYMBOL_TEXT_LINES);
	LoadSymbol ((SYMBOL_OHM_HOME + 0x160), SymbolOhm, SYMBOL_TEXT_WIDTH, SYMBOL_TEXT_LINES);
	//
	WriteSettings (AUTO_MODE, AUTOHz); 
	LoadSeries(DASH_SYM, DASH_SYM);

	WriteOutput ();	//   ,    

}


//////////////////////////////////////////


int LoadSymbol (int HOME_ADDRESS, unsigned char SYMBOL[], int SYMBOL_WIDTH, int SYMBOL_LINES) // 
{

	// 
	int	Address;
	int MemCount;	
	int	Lines;
	Lines = SYMBOL_LINES;	//{} 
	Address=0;

	while (Lines > 0)
	{
		while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS  {} 
		WriteData(LOBYTE(HOME_ADDRESS + ((SYMBOL_LINES - Lines)*22)));
		while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
		WriteData(HIBYTE(HOME_ADDRESS + ((SYMBOL_LINES - Lines)*22)));
		while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
		WriteCommand(0x24); 
	
		//  
		while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
		WriteCommand(0xB0); 	//START AUTOMODE  

		MemCount=(SYMBOL_WIDTH);
	
		while  (MemCount!=0)
		{	
			while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
	
			WriteData(SYMBOL[Address]); 
			MemCount--;
			Address++;
		}	
		while ( (CheckStatus() & 0x08) != 0x08){}	//WAIT FOR STATUS BITS
		WriteCommand(0xB2); 	//END AUTOMODE  
	
		Lines--;
	}

return 1;
}


//////////////////////////////////////////


void LoadParallel (int Primary, int Secondary) //Loads Screen  
{

	//  
	ClearCircuitArea();
	
	//Load Paralell Branch (right) {} Paralell ()
	LoadSymbol (SYMBOL_PARALEL_RIGHT_HOME, SymbolParalelRight, SYMBOL_PARALEL_WIDTH, SYMBOL_PARALEL_LINES);

	//Load Paralell Branch (left) () {} Paralell
	LoadSymbol (SYMBOL_PARALEL_LEFT_HOME, SymbolParalelLeft, SYMBOL_PARALEL_WIDTH, SYMBOL_PARALEL_LINES);

	//Load Primary Element (Upper)   () 
	if (Primary == CAPACITOR_SYM)
	{
		LoadSymbol (PARALELL_TOP_HOME, SymbolCapacitor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Primary == INDUCTOR_SYM)
	{
		LoadSymbol (PARALELL_TOP_HOME, SymbolInductor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else
	{
		LoadSymbol (PARALELL_TOP_HOME, SymbolResistor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}

	//Load Secondary Element (Lower)   ()
	if (Secondary == CAPACITOR_SYM)
	{
		LoadSymbol (PARALELL_BOTTOM_HOME, SymbolCapacitor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Secondary == INDUCTOR_SYM)
	{
		LoadSymbol (PARALELL_BOTTOM_HOME, SymbolInductor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else
	{
		LoadSymbol (PARALELL_BOTTOM_HOME, SymbolResistor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}

}


//////////////////////////////////////////


int LoadSeries (int Primary, int Secondary) //Loads Series Circuit Screen {  }  
{

	//Clear Screen  
	ClearCircuitArea();
	
	//Load Primary Element (Upper)  () 
	if (Primary == CAPACITOR_SYM)
	{
		LoadSymbol (SERIES_LEFT_HOME, SymbolCapacitor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Primary == INDUCTOR_SYM)
	{
		LoadSymbol (SERIES_LEFT_HOME, SymbolInductor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Primary == RESISTOR_SYM)
	{
		LoadSymbol (SERIES_LEFT_HOME, SymbolResistor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else 
	{
		LoadSymbol (SERIES_LEFT_HOME, SymbolDashLine, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}

	//Load Secondary Element (Lower)    ()
	if (Secondary == CAPACITOR_SYM)
	{
		LoadSymbol (SERIES_RIGHT_HOME, SymbolCapacitor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Secondary == INDUCTOR_SYM)
	{
		LoadSymbol (SERIES_RIGHT_HOME, SymbolInductor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else if (Secondary == RESISTOR_SYM)
	{
		LoadSymbol (SERIES_RIGHT_HOME, SymbolResistor, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}
	else 
	{
		LoadSymbol (SERIES_RIGHT_HOME, SymbolDashLine, CIRCUIT_ELEMENT_WIDTH, CIRCUIT_ELEMENT_LINES);
	}

return 1;
}


//////////////////////////////////////////


int ClearCircuitArea () //Clears Circuit Screen Area    {  }
{

	// 
	int MemCount;	
	int	Lines;
	Lines = 32;

	while (Lines > 0)
	{
		while ( (CheckStatus() & 0x03) != 0x03){}				//WAIT FOR STATUS BITS {} 
		WriteData(LOBYTE(CIRCUIT_AREA_HOME+ ((32 - Lines)*22)));
		while ( (CheckStatus() & 0x03) != 0x03){}				//WAIT FOR STATUS BITS
		WriteData(HIBYTE(CIRCUIT_AREA_HOME+ ((32 - Lines)*22)));
		while ( (CheckStatus() & 0x03) != 0x03){}				//WAIT FOR STATUS BITS
		WriteCommand(0x24); 
	
		// Auto Mode  
		while ( (CheckStatus() & 0x03) != 0x03){}				//WAIT FOR STATUS BITS
		WriteCommand(0xB0); 									//START AUTOMODE  
	
		MemCount=(6);
	
		while  (MemCount!=0)
		{	
			while ( (CheckStatus() & 0x08) != 0x08){}			//WAIT FOR STATUS BITS
	
			WriteData(0X00); 
			MemCount--;
		}	
		while ( (CheckStatus() & 0x08) != 0x08){}				//WAIT FOR STATUS BITS
		WriteCommand(0xB2); 									//END AUTOMODE  
		Lines--;
	}

return 1;
}


//////////////////////////////////////////


int WriteSettings (int Mode, int Freq) //Writes Settings to Screen  {},    
{

		if (Mode == AUTO_MODE)
		{
			WriteText(MODE_HOME, MsgAuto, MODE_CHAR);	
		}
		else if (Mode == SERIES_MODE)
		{
			WriteText(MODE_HOME, MsgSeries, MODE_CHAR);
		}
		else if (Mode == PARALLEL_MODE)
		{
			WriteText(MODE_HOME, MsgParallel, MODE_CHAR);
		}
		else if (Mode == TESTING_MODE)
		{
			WriteText(MODE_HOME, MsgTesting, MODE_CHAR);	
		}
		else if (Mode == CALIBRATION_MODE)
		{
			WriteText(MODE_HOME, MsgRun_Cal, MODE_CHAR);
		}
		else
		{
			WriteText(MODE_HOME, MsgBlank, MODE_CHAR);
		}

		//
	
		if (Freq == AUTOHz)
		{
			WriteText(FREQ_HOME, FreqAUTOHz, FREQ_CHAR);
		}
		else if (Freq == F100Hz)
		{
			WriteText(FREQ_HOME, Freq100Hz, FREQ_CHAR);
		}
		else if (Freq == F1KHz)
		{
			WriteText(FREQ_HOME, Freq1KHz, FREQ_CHAR);
		}
		else if (Freq == F10KHz)
		{
			WriteText(FREQ_HOME, Freq10KHz, FREQ_CHAR);
		}
		else
		{
			WriteText(FREQ_HOME, MsgBlank, FREQ_CHAR);
		}

return 1;
}


//////////////////////////////////////////


int WriteOutput () //Writes Settings to Screen  {},    
{

	// 
	int MemCount;
	int Temp;

	// (Z)-
	BUF_OUT[1]= 'Z';
	BUF_OUT[0]= ' ';
	StringNumMake (LCR_MEASUREMENTS.IMPEDANCE);
	BUF_OUT[8] = ' ';
	WriteText ( DATA_NUM_HOME , BUF_OUT, 9);
	
	// (R)-
	BUF_OUT[1]= 'R';
	BUF_OUT[0]= ' ';
	StringNumMake (LCR_MEASUREMENTS.RESISTANCE);
	BUF_OUT[8] = ' ';
	WriteText ( DATA_NUM_HOME + 22 , BUF_OUT, 9);

	// ()-  ( )
	BUF_OUT[1]= 'X';
	BUF_OUT[0]= ' ';
	StringNumMake (LCR_MEASUREMENTS.COMPLEX_RES);
	BUF_OUT[8] = ' ';
	WriteText ( DATA_NUM_HOME + 44 , BUF_OUT, 9);

	if (LCR_MEASUREMENTS.CAPINDTYPE == CAP )
	{

		// (C)-
		BUF_OUT[1]= 'C';
		BUF_OUT[0]= ' ';
		StringNumMake (LCR_MEASUREMENTS.CAP_IND);
		BUF_OUT[8] = 'F';
		WriteText ( DATA_NUM_HOME + 66 , BUF_OUT, 9);

		// (D)-Disapation Factor  Disapation
		BUF_OUT[1]= 'D';
		BUF_OUT[0]= ' ';
		StringNumMake (LCR_MEASUREMENTS.QUAL_DISAP);
		BUF_OUT[8] = ' ';
		WriteText ( DATA_NUM_HOME + 88 , BUF_OUT, 9);

	}

	else
	{

		// (L)-
		BUF_OUT[1]= 'L';
		BUF_OUT[0]= ' ';
		StringNumMake (LCR_MEASUREMENTS.CAP_IND);
		BUF_OUT[8] = 'H';
		WriteText ( DATA_NUM_HOME + 66 , BUF_OUT, 9);

		// (Q)- ( )
		BUF_OUT[1]= 'Q';
		BUF_OUT[0]= ' ';
		StringNumMake (LCR_MEASUREMENTS.QUAL_DISAP);
		BUF_OUT[8] = ' ';
		WriteText ( DATA_NUM_HOME + 88 , BUF_OUT, 9);

	}

	// (F)-Phase Angle  
	BUF_OUT[1]= ' ';
	BUF_OUT[0]= ' ';
	//Fix Situation of Small Angle Display (Avoid m,u,p,n etc)      ( ., u, p, n  .), 
	if	(LCR_MEASUREMENTS.PHASE_ANGLE < 0)
	{
		if (LCR_MEASUREMENTS.PHASE_ANGLE > (-1)) 
		{
			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;

			BUF_OUT[2] = '-';
			BUF_OUT[3] = ' ';
			BUF_OUT[4] = '0';
			BUF_OUT[5] = '.';
			Temp = LCR_MEASUREMENTS.PHASE_ANGLE * 10;
			if (Temp == 0) //Zero always Positive ,  
			{
				BUF_OUT[2] = '+';
			}
			BUF_OUT[6]= Temp + 0x30;

			BUF_OUT[7] = ' ';

			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;
		}

		else if (LCR_MEASUREMENTS.PHASE_ANGLE < (-90))
		{
			LCR_MEASUREMENTS.PHASE_ANGLE = -90;
			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;
			StringNumMake (LCR_MEASUREMENTS.PHASE_ANGLE);
			BUF_OUT[2] = '-';
			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;
		}

		else
		{
			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;
			StringNumMake (LCR_MEASUREMENTS.PHASE_ANGLE);
			BUF_OUT[2] = '-';
			LCR_MEASUREMENTS.PHASE_ANGLE = LCR_MEASUREMENTS.PHASE_ANGLE * -1;
		}

	}

	else
	{
		if (LCR_MEASUREMENTS.PHASE_ANGLE < (1)) 
		{
			BUF_OUT[2] = '+';
			BUF_OUT[3] = ' ';
			BUF_OUT[4] = '0';
			BUF_OUT[5] = '.';
			Temp = LCR_MEASUREMENTS.PHASE_ANGLE * 10;
			BUF_OUT[6]= Temp + 0x30;

			BUF_OUT[7] = ' ';
		}

		else if (LCR_MEASUREMENTS.PHASE_ANGLE > 90)
		{
			LCR_MEASUREMENTS.PHASE_ANGLE =  90;
			StringNumMake (LCR_MEASUREMENTS.PHASE_ANGLE);
			BUF_OUT[2] = '+';
		}

		else
		{
			StringNumMake (LCR_MEASUREMENTS.PHASE_ANGLE);
			BUF_OUT[2] = '+';
		}

	}

	if (LCR_MEASUREMENTS.PHASE_ANGLE == 0)
	{
		StringNumMake (LCR_MEASUREMENTS.PHASE_ANGLE);
		BUF_OUT[2] = ' ';
	}

	BUF_OUT[8] = ' ';
	WriteText ( DATA_NUM_HOME + 110 , BUF_OUT, 9);

return 1;
}


//////////////////////////////////////////


int WriteText (int HomeAddress, unsigned char Text[],  int NumBytes) //Writes Settings to Screen   {},    
{

	// 
	int	Address;
	int MemCount;
	Address = 0;

	while ( (CheckStatus() & 0x03) != 0x03){}		//WAIT FOR STATUS BITS  {} 
	WriteData(LOBYTE(TEXT_HOME_SET + HomeAddress));
	while ( (CheckStatus() & 0x03) != 0x03){}		//WAIT FOR STATUS BITS
	WriteData(HIBYTE(TEXT_HOME_SET + HomeAddress));
	while ( (CheckStatus() & 0x03) != 0x03){}		//WAIT FOR STATUS BITS
	WriteCommand(0x24); 
		
	MemCount=(NumBytes);
	
	while  (MemCount!=0)
	{	
		WriteData((Text[Address]) - 0x20);	
		while ( (CheckStatus() & 0x03) != 0x03){}	//WAIT FOR STATUS BITS
		WriteCommand(0XC0); 
		Address++;
		MemCount--;
	}

return 1;	
}


//////////////////////////////////////////


int StringNumMake (float Number) //Writes Settings to Screen   {},    
{

	// 
	int Temp;

	if (Number >= 1000000)
	{
		BUF_OUT[7]= 'M';
		if (Number > 999900000)
		{
		Number = 999900000;		//Overload Protection  
		}

		Number = Number / 1000000;
	
	}

	else if(Number >= 1000)
	{	
		BUF_OUT[7]= 'K';
		Number = Number / 1000;
	}

	else if (Number >= 1)
	{
		BUF_OUT[7]= ' ';
	}

	else if (Number >= 0.001)
	{
		BUF_OUT[7]= 'm';
		Number = Number *1000;
	}

	else if (Number >= 0.000001)
	{
		BUF_OUT[7]= 'u';
		Number = Number * 1000000;
	}

	else if (Number >= 0.000000001)
	{
		BUF_OUT[7]= 'n';
		Number = Number * 1000000000;
	}

	else if (Number >= (0.000000001 / 10000))
	{
		BUF_OUT[7]= 'p';
		Number = Number * 1000000 * 1000000;
	}

	else // FOR NO VALUE SITUATION     
	{
		BUF_OUT[7]= ' ';
		Number = Number;
	}

	//  
	Temp = Number / 100;
	if	(Temp > 0)
	{
		BUF_OUT[2]= Temp + 0x30;
    }

	else
	{
		BUF_OUT[2] = ' ' ;
	}
	
	Number = Number - (Temp * 100);

	//  
	Temp = Number / 10;
	if	(Temp > 0)
	{
		BUF_OUT[3]= Temp + 0x30;
    }

	else if (BUF_OUT[2] == ' ')
	{
		BUF_OUT[3] = ' ' ;
	}

	else 
	{
		BUF_OUT[3] = '0' ;
	}

	Number = Number - (Temp * 10);

	//  
	Temp = Number / 1;
	BUF_OUT[4]= Temp + 0x30;

	Number = Number - (Temp * 1);

	//  (.)
	BUF_OUT[5]= '.';

	//  
	Temp = Number *10;
	BUF_OUT[6]= Temp + 0x30;

return 1;
}


/********************************************************************/
