/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        LCR METER FUNCS.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* Main LCR Functions For DSP LCR METER
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include "dsp.h"
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"
	#include "Graphic LCD.h"
	#include "SPI GPIO.h"
	#include "SPI DDS.h"
	#include "SPI MCP PGA.h"
	#include "SPI MCP POT.h"
	#include "LCR METER INIT.h"

/*****************************************************************************/
//DEFINES

	#define _10K_NUM_SAMPLES 	80
	#define _1K_NUM_SAMPLES 	50
	#define _100_NUM_SAMPLES 	96

	#define	UPFREQ				24000000

	#define	CAP					0x00
	#define	IND					0x01

	#define	F10KHz				0x03	
	#define	F1KHz				0x02
	#define	F100Hz				0x01
	#define	AUTOHz				0x00

	#define	CALIBRATION_MODE 	0x04
	#define	TESTING_MODE 		0x03
	#define	PARALLEL_MODE		0x02
	#define	SERIES_MODE 		0x01
	#define	AUTO_MODE			0x00

	#define SCREEN_BLANK 		0x00
	#define	SCREEN_RESULT		 0x01

	#define	DASH_SYM			0x00
	#define	RESISTOR_SYM 		0x01
	#define	CAPACITOR_SYM 		0x02
	#define	INDUCTOR_SYM		0x03

	#define	SW_A				LATCbits.LATC13  
	#define	SW_V				LATCbits.LATC14  

	#define	Negative			0
	#define	Positive			1

	#define GainLimit  			0.4  //(1v above offset)


/*****************************************************************************/
//GLOBAL VARIABLES

unsigned char MODE_SET;
unsigned char FREQ_SET;

unsigned char SCREEN_SET;

//
struct {

	unsigned char MODE;
	unsigned char FREQ;

	unsigned char CAPINDTYPE;

	unsigned char DOMINANT_ELEMENT;
	unsigned char SECONDARY_ELEMENT;

	float	IMPEDANCE;
	float	RESISTANCE;
	float	COMPLEX_RES;
	float	CAP_IND;
	float	QUAL_DISAP;
	float 	PHASE_ANGLE;

} LCR_MEASUREMENTS;
//

float	Z10K;
float	THETA10K;

float	Z1K;
float	THETA1K;

float	Z100;
float	THETA100;

fractional	VoltageMeas[96];
fractional	CurrentMeas[96];

fractional	FilteredVoltageMeas[96];
fractional	FilteredCurrentMeas[96];

fractional	DecimateResult[2];


/*****************************************************************************/


void INIT_LCR_FUNCS()
{

	MODE_SET = AUTO_MODE;
	FREQ_SET = AUTOHz;
	
	SCREEN_SET = SCREEN_BLANK; //Initialize Values
	
	LCR_MEASUREMENTS.MODE = 0;
	LCR_MEASUREMENTS.FREQ = 0;
	LCR_MEASUREMENTS.CAPINDTYPE = 0;
	LCR_MEASUREMENTS.DOMINANT_ELEMENT = 0;
	LCR_MEASUREMENTS.SECONDARY_ELEMENT = 0;
	LCR_MEASUREMENTS.IMPEDANCE = 0;
	LCR_MEASUREMENTS.RESISTANCE = 0;
	LCR_MEASUREMENTS.COMPLEX_RES = 0;
	LCR_MEASUREMENTS.CAP_IND = 0;
	LCR_MEASUREMENTS.QUAL_DISAP = 0;
	LCR_MEASUREMENTS.PHASE_ANGLE = 0;
	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

}


/////////////////////////////////////////////////////////////////


void CHANGE_MODE()
{

	if (SCREEN_SET == SCREEN_RESULT)
	{

		//Create Clear Screen
		INIT_LCR_FUNCS();
		WriteOutput ();		//Write Zero Data to Screen
		//WriteSettings (AUTO_MODE, AUTOHz); 
		LoadSeries(DASH_SYM, DASH_SYM);

		SCREEN_SET = SCREEN_BLANK;

	}

	else
	{

		if (MODE_SET == 2)
		{
			MODE_SET = 0;
		}

		else
		{
			MODE_SET++;
		}
	
	}

	WriteSettings (MODE_SET, FREQ_SET); 
}


/////////////////////////////////////////////////////////////////


void CHANGE_FREQ()
{

	if (SCREEN_SET == SCREEN_RESULT)
	{

		//Create Clear Screen
		INIT_LCR_FUNCS();
		WriteOutput ();	//Write Zero Data to Screen
		LoadSeries(DASH_SYM, DASH_SYM);

		SCREEN_SET = SCREEN_BLANK;

	}

	else
	{

		if (FREQ_SET == 3)
		{
			FREQ_SET = 0;
		}

		else
		{
			FREQ_SET++;
		}
	
	}
		WriteSettings (MODE_SET, FREQ_SET); 

}


/////////////////////////////////////////////////////////////////


void RUN_TEST()
{

	if (SCREEN_SET == SCREEN_RESULT)
	{

		//Create Clear Screen
		INIT_LCR_FUNCS();
		WriteOutput ();	//Write Zero Data to Screen
		WriteSettings (MODE_SET, FREQ_SET); 
		LoadSeries(DASH_SYM, DASH_SYM);

		SCREEN_SET = SCREEN_BLANK;

	}

	else //Run Test
	{

		//Write "TESTING" to Screen
		WriteSettings (TESTING_MODE, 0xFF); 
		//RUN TEST

		if (FREQ_SET == F10KHz)
		{

			TEST_10K(10);	
			LCR_MEASUREMENTS.IMPEDANCE = Z10K;
			LCR_MEASUREMENTS.PHASE_ANGLE = THETA10K;

			LCR_MEASUREMENTS.FREQ = F10KHz;

		}

		else if (FREQ_SET == F1KHz)
		{

			TEST_1K(5);
			LCR_MEASUREMENTS.IMPEDANCE = Z1K;
			LCR_MEASUREMENTS.PHASE_ANGLE = THETA1K;

			LCR_MEASUREMENTS.FREQ = F1KHz;
		}

		else if (FREQ_SET == F100Hz)
		{

			TEST_100(5);
			LCR_MEASUREMENTS.IMPEDANCE = Z100;
			LCR_MEASUREMENTS.PHASE_ANGLE = THETA100;

			LCR_MEASUREMENTS.FREQ = F100Hz;

		}

		else //Auto Frequency Mode
		{

			//Test for base Impedance
			TEST_10K(1);
			TEST_1K(1);
			TEST_100(1);

			//Re-test (Average) with apropriate frequency
			if ( THETA1K < 0.5 ) //Resistor Reading Use Default Frequency 1KHz
			{		

				TEST_1K(5);
				LCR_MEASUREMENTS.IMPEDANCE = Z1K;
				LCR_MEASUREMENTS.PHASE_ANGLE = THETA1K;
	
				LCR_MEASUREMENTS.FREQ = F1KHz;

			}

			else
			{

				if  ( ( (fabs(Z100 - 1000)) <  (fabs(Z10K - 1000)) ) && ( (fabs(Z100 - 1000)) < (fabs(Z1K - 1000)) ) )
				{

					TEST_100(5);
					LCR_MEASUREMENTS.IMPEDANCE = Z100;
					LCR_MEASUREMENTS.PHASE_ANGLE = THETA100;
		
					LCR_MEASUREMENTS.FREQ = F100Hz;

				}

				else if ( ( (fabs(Z10K - 1000)) <  (fabs(Z100 - 1000)) ) && ( (fabs(Z10K - 1000)) < (fabs(Z1K - 1000)) ) )
				{

					TEST_10K(10);	
					LCR_MEASUREMENTS.IMPEDANCE = Z10K;
					LCR_MEASUREMENTS.PHASE_ANGLE = THETA10K;
		
					LCR_MEASUREMENTS.FREQ = F10KHz;

				}

				else	//Frequency = 1K (Default)
				{

					TEST_1K(5);
					LCR_MEASUREMENTS.IMPEDANCE = Z1K;
					LCR_MEASUREMENTS.PHASE_ANGLE = THETA1K;
					LCR_MEASUREMENTS.FREQ = F1KHz;

				}	

			}

		}

		if	(LCR_MEASUREMENTS.PHASE_ANGLE < -90)	
		{
			LCR_MEASUREMENTS.PHASE_ANGLE =  - 90;
		}

		else if (LCR_MEASUREMENTS.PHASE_ANGLE > 90)
		{
			LCR_MEASUREMENTS.PHASE_ANGLE =  90;
		}


		
		if (MODE_SET == AUTO_MODE)
		{

			if (LCR_MEASUREMENTS.PHASE_ANGLE < 0) //Capacitor 
			{
				//Asume Paralell & check capacitance
				LCR_MEASUREMENTS.COMPLEX_RES = 1 / ( sinf((fabs(LCR_MEASUREMENTS.PHASE_ANGLE)/180) * PI) * (1 / LCR_MEASUREMENTS.IMPEDANCE) );

				if (LCR_MEASUREMENTS.FREQ == F10KHz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 10 * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else if (LCR_MEASUREMENTS.FREQ == F100Hz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 100 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else //1KHz
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}
			
				if (LCR_MEASUREMENTS.CAP_IND <= (0.0000000001 )) //Smaller than 100pF?
				{
					MODE_SET = PARALLEL_MODE; //Small CAP - Use Parallel Mode.
				}

				else
				{
					MODE_SET = SERIES_MODE; //Always Series Unless Small CAP.
				}

			}

			else	
			{
				MODE_SET = SERIES_MODE; //Always Series Unless Small CAP.
			}			

		}

		
		if (fabs(LCR_MEASUREMENTS.PHASE_ANGLE) < 0.1) //Check For Resistor Only
		{
			LCR_MEASUREMENTS.MODE = SERIES_MODE ;

			LCR_MEASUREMENTS.CAPINDTYPE = 0;
			LCR_MEASUREMENTS.RESISTANCE = LCR_MEASUREMENTS.IMPEDANCE;
			LCR_MEASUREMENTS.COMPLEX_RES = 0;
			LCR_MEASUREMENTS.CAP_IND = 0;
			LCR_MEASUREMENTS.QUAL_DISAP = 0;
			//Both Elements RESISTORS
			LCR_MEASUREMENTS.DOMINANT_ELEMENT = RESISTOR_SYM ;
			LCR_MEASUREMENTS.SECONDARY_ELEMENT = RESISTOR_SYM ;

			LoadSeries(LCR_MEASUREMENTS.DOMINANT_ELEMENT , LCR_MEASUREMENTS.SECONDARY_ELEMENT);
		
		}
		else if (MODE_SET == SERIES_MODE)
		{
			LCR_MEASUREMENTS.MODE = SERIES_MODE ;

			LCR_MEASUREMENTS.COMPLEX_RES = sinf((fabs(LCR_MEASUREMENTS.PHASE_ANGLE)/180) * PI) * LCR_MEASUREMENTS.IMPEDANCE;
			LCR_MEASUREMENTS.RESISTANCE = cosf((fabs(LCR_MEASUREMENTS.PHASE_ANGLE)/180) * PI) * LCR_MEASUREMENTS.IMPEDANCE;	
			
			if (LCR_MEASUREMENTS.PHASE_ANGLE < 0) //Capacitor
			{
				LCR_MEASUREMENTS.CAPINDTYPE = CAP;
				LCR_MEASUREMENTS.QUAL_DISAP = LCR_MEASUREMENTS.RESISTANCE / LCR_MEASUREMENTS.COMPLEX_RES;

				if (LCR_MEASUREMENTS.FREQ == F10KHz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 10 * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else if (LCR_MEASUREMENTS.FREQ == F100Hz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 100 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else //1KHz
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				if (LCR_MEASUREMENTS.COMPLEX_RES > LCR_MEASUREMENTS.RESISTANCE) //Capacitor is Primary Component
				{
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = CAPACITOR_SYM ;
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = RESISTOR_SYM ;
				}

				else //Resitor is Dominant
				{
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = RESISTOR_SYM ;
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = CAPACITOR_SYM ;		
				}

			}

			else //Inductor
			{
				LCR_MEASUREMENTS.CAPINDTYPE = IND;
				LCR_MEASUREMENTS.QUAL_DISAP = LCR_MEASUREMENTS.COMPLEX_RES / LCR_MEASUREMENTS.RESISTANCE;
				if (LCR_MEASUREMENTS.FREQ == F10KHz)
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 10 * 1000 );
				}

				else if (LCR_MEASUREMENTS.FREQ == F100Hz)
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 100);
				}

				else //1KHz
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 1000 );
				}

				if (LCR_MEASUREMENTS.COMPLEX_RES > LCR_MEASUREMENTS.RESISTANCE) //Inductor is Primary Component
				{
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = INDUCTOR_SYM ;
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = RESISTOR_SYM ;
				}

				else //Resitor is Dominant
				{
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = RESISTOR_SYM ;
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = INDUCTOR_SYM ;		
				}
			}

			LoadSeries(LCR_MEASUREMENTS.DOMINANT_ELEMENT , LCR_MEASUREMENTS.SECONDARY_ELEMENT);

		}
		else // Parallel Mode
		{

			LCR_MEASUREMENTS.MODE = PARALLEL_MODE ;

			LCR_MEASUREMENTS.COMPLEX_RES = 1 / ( sinf((fabs(LCR_MEASUREMENTS.PHASE_ANGLE)/180) * PI) * (1 / LCR_MEASUREMENTS.IMPEDANCE) );
			LCR_MEASUREMENTS.RESISTANCE = 1 / ( cosf((fabs(LCR_MEASUREMENTS.PHASE_ANGLE)/180) * PI) * (1 / LCR_MEASUREMENTS.IMPEDANCE) );	
			
			if (LCR_MEASUREMENTS.PHASE_ANGLE < 0) //Capacitor
			{
				LCR_MEASUREMENTS.CAPINDTYPE = CAP;
				LCR_MEASUREMENTS.QUAL_DISAP = LCR_MEASUREMENTS.COMPLEX_RES / LCR_MEASUREMENTS.RESISTANCE;
				if (LCR_MEASUREMENTS.FREQ == F10KHz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 10 * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else if (LCR_MEASUREMENTS.FREQ == F100Hz)
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 100 * LCR_MEASUREMENTS.COMPLEX_RES);
				}

				else //1KHz
				{
					LCR_MEASUREMENTS.CAP_IND = 1 / (2 * PI * 1000 * LCR_MEASUREMENTS.COMPLEX_RES);
				}


				if (LCR_MEASUREMENTS.COMPLEX_RES > LCR_MEASUREMENTS.RESISTANCE) //Capacitor is Primary Component
				{
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = CAPACITOR_SYM ;
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = RESISTOR_SYM ;
				}

				else //Resitor is Larger
				{
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = RESISTOR_SYM ;
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = CAPACITOR_SYM ;		
				}

			}

			else //Inductor
			{
				LCR_MEASUREMENTS.CAPINDTYPE = IND;
				LCR_MEASUREMENTS.QUAL_DISAP = LCR_MEASUREMENTS.RESISTANCE / LCR_MEASUREMENTS.COMPLEX_RES ;
				if (LCR_MEASUREMENTS.FREQ == F10KHz)
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 10 * 1000 );
				}

				else if (LCR_MEASUREMENTS.FREQ == F100Hz)
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 100);
				}

				else //1KHz
				{
					LCR_MEASUREMENTS.CAP_IND = LCR_MEASUREMENTS.COMPLEX_RES / (2 * PI * 1000 );
				}


				if (LCR_MEASUREMENTS.COMPLEX_RES > LCR_MEASUREMENTS.RESISTANCE) //Inductor is Primary Component
				{
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = INDUCTOR_SYM ;
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = RESISTOR_SYM ;
				}

				else //Resitor is Larger
				{
					LCR_MEASUREMENTS.SECONDARY_ELEMENT = RESISTOR_SYM ;
					LCR_MEASUREMENTS.DOMINANT_ELEMENT = INDUCTOR_SYM ;		
				}

			}

			LoadParallel(LCR_MEASUREMENTS.DOMINANT_ELEMENT , LCR_MEASUREMENTS.SECONDARY_ELEMENT);

		}

		WriteSettings (LCR_MEASUREMENTS.MODE, LCR_MEASUREMENTS.FREQ);	
		WriteOutput ();	
		SCREEN_SET = SCREEN_RESULT;

	}
	
}


/////////////////////////////////////////////////////////////////


void RUN_CAL()
{
	if (SCREEN_SET == SCREEN_RESULT)
	{

		//Create Clear Screen
		INIT_LCR_FUNCS();
		WriteOutput ();	//Write Zero Data to Screen
		WriteSettings (MODE_SET, FREQ_SET);  
		LoadSeries(DASH_SYM, DASH_SYM);

		SCREEN_SET = SCREEN_BLANK;

	}

	else
	{

		WriteSettings (CALIBRATION_MODE, 0xFF); 

		//Perform ADC Offset Adjustment
		RUN_OFFSET();

		GenDelay(1000, UPFREQ);		//Wait 1Sec.

		//Create Clear Screen
		INIT_LCR_FUNCS();
		WriteOutput ();		//Write Zero Data to Screen
		//WriteSettings (AUTO_MODE, AUTOHz); 
		LoadSeries(DASH_SYM, DASH_SYM);

		SCREEN_SET = SCREEN_BLANK;


	}
}


/////////////////////////////////////////////////////////////////


int RUN_OFFSET() //Adjusts Digital Pot For Zero Offset (Connect CAL Res)
{

	//Local Variables
	int		POTValue;
	float	OLDValue;
	float	NEWValue;
	int		POTSetMid;
	int		Count;

	//Set DDS Output To Mid Value
	RESET_DDS_SPI();
	
	//Set High Gain
	SW_V = 1;
	SET_SPI_PGA_GAIN(16, 'V');

	//Perform ADC Offset Adjustment
	POTValue = 0;
	SET_SPI_MCP_POT(POTValue);
	GenDelay(100, UPFREQ);

	set_ANALOG_OFFSET();		//Set ADC for Offset Adjust

	Count = 0;
	while (Count < 5)
	{
		ADCON1bits.SAMP = 1; 	// start sampling then ...
		// after xTad go to conversion
		while (!ADCON1bits.DONE); // conversion done?
		OLDValue = OLDValue + Fract2Float(ADCBUF0); // yes then get ADC value
		Count++;
	}

	OLDValue = OLDValue / 10;

	POTSetMid = 0;

	while (POTValue <= 128)
	{
		SET_SPI_MCP_POT(POTValue);
		GenDelay(50, UPFREQ);

		Count = 0;
		NEWValue = 0;

			while (Count < 5)
			{	
				ADCON1bits.SAMP = 1; // start sampling then ...
				// after xTad go to conversion
				while (!ADCON1bits.DONE); // conversion done?
				NEWValue = NEWValue + Fract2Float(ADCBUF0); // yes then get ADC value
			Count++;

			}
	
		NEWValue = NEWValue / 10;	

		if (fabs(NEWValue) < fabs(OLDValue))
		{
			OLDValue = NEWValue;
			POTSetMid = POTValue;			
		}

		POTValue++;
	}

	SET_SPI_MCP_POT(POTSetMid);
	GenDelay(200, UPFREQ);

	GenDelay(1000, UPFREQ);
	//Set Low Gain
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');

}


/////////////////////////////////////////////////////////////////


int TEST_10K(int Average)
{

	extern IIRCanonicStruct IIR_BP_10KFilter;

	//Local Variables
	float VHigh;
	float AHigh;
	int	VGain;
	int	AGain;
	int TEST_FINISH;
	int AddPointer;
	int	AvCount;
	int	ZeroCrossV;
	int	ZeroCrossA;
	int ZeroCompare;
	float ZeroOffsetV;
	float ZeroOffsetA;
	float PeakV;
	float PeakA;
	float OffsetShift;
	float OffsetTemp;
	float CrossingError;
	float CrossingErrorMin;
	float CrossingGuess;
	float CrossingTemp;

	//Initialize Locals	
	VHigh = 0;
	AHigh = 0;
	AGain = 1;
	VGain = 1;
	TEST_FINISH = 0;
	AddPointer = 0;

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

	//Set ADC 	
	set_ANALOG_LCR();

	//Set DDS Frequency
	SET_FREQ_DDS_SPI (F10KHz);
	RUN_DDS_SPI();
	GenDelay(50, UPFREQ);  // Settle DDS Output & Allow ADC Settle

	//Run Test
	while	(TEST_FINISH == 0)
	{

		GenDelay(25, UPFREQ);	//Allow New Gain Settle

		AddPointer = 0; //Reset Pointer
		IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
		ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling

		while (AddPointer < _10K_NUM_SAMPLES)  //Collect Sample Set
		{		

			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			VoltageMeas[AddPointer] = ADCBUF0;
			CurrentMeas[AddPointer] = ADCBUF1;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUF2;
			CurrentMeas[AddPointer] = ADCBUF3;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUF4;
			CurrentMeas[AddPointer] = ADCBUF5;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUF6;
			CurrentMeas[AddPointer] = ADCBUF7;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUF8;
			CurrentMeas[AddPointer] = ADCBUF9;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUFA;
			CurrentMeas[AddPointer] = ADCBUFB;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUFC;
			CurrentMeas[AddPointer] = ADCBUFD;
			AddPointer ++;
			VoltageMeas[AddPointer] = ADCBUFE;
			CurrentMeas[AddPointer] = ADCBUFF;
			AddPointer ++;

		}
	
		ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling

		//
		IIRCanonicInit (&IIR_BP_10KFilter); //Clear Filter Histroy
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (_10K_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_10KFilter  );
			AddPointer ++;
		}

		//
		IIRCanonicInit (&IIR_BP_10KFilter);
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (_10K_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_10KFilter  );
			AddPointer ++;
		}

		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			//Fix Voltage Offset
			OffsetShift =( ( Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2))])  + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 1)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 2)])  + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 3)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+4)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+5)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+6)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+7)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+8)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+9)])  ) / 5);
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _10K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Voltage Gain
			AddPointer = 29; //Reset Pointer
			while (AddPointer < 50)
			{		
				if ((Fract2Float(FilteredVoltageMeas[AddPointer])) > VHigh)
				{
					VHigh = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			VGain = VGain * 2;

			if (VGain >= 64)
			{
				SW_V = 1;
				SET_SPI_PGA_GAIN((VGain/64), 'V');
			}

			else
			{
				SW_V = 0;
				SET_SPI_PGA_GAIN((VGain), 'V');
			}
		}

		if ((AHigh < GainLimit) && (AGain < 2048))
		{

			//Fix Current Offset
			OffsetShift =( ( Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2))])  + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 1)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 2)])  + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 3)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+4)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+5)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+6)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+7)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+8)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+9)])  ) / 5);
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;

			while (AddPointer < _10K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Current Gain
			AddPointer = 29; //Reset Pointer

			while (AddPointer < 50)
			{		
				if ( (Fract2Float(FilteredCurrentMeas[AddPointer])) > AHigh)
				{
					AHigh = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((AHigh < GainLimit) && (AGain < 2048))
		{
			AGain = AGain * 2;

			if (AGain >= 64)
			{
				SW_A = 1;
				SET_SPI_PGA_GAIN((AGain/64), 'A');
			}

			else
			{
				SW_A = 0;
				SET_SPI_PGA_GAIN((AGain), 'A');
			}
		}

		//Check if conditions are met to abort test	
		if (((VHigh >= GainLimit)&&(AHigh >= GainLimit)) || ((VGain == 2048)&&(VHigh < GainLimit)) || ((AGain == 2048)&&(AHigh < GainLimit)) )	
		{
			TEST_FINISH = 1; //Abort Test
		}

	}

	//Check Case Of Minimum Impedance
	if ((VGain == 2048)&&(VHigh < (GainLimit / 4)))
	{
		Z10K = 0;
		THETA10K = 0;	
	}	
	//Check Case Of Maximum Impedance

	else if ((AGain == 2048)&&(AHigh < (GainLimit / 4)))
	{
		Z10K = 999900 * 1000;
		THETA10K = 0;	
	}	

	// Impedance in range
	else 
	{
		THETA10K = 0;	
		Z10K = 0;
		AvCount = 0;

		while (AvCount < Average)
		{
			// Make Reading
			AddPointer = 0; //Reset Pointer
			IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
			ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling

			while (AddPointer < _10K_NUM_SAMPLES)  //Collect Sample Set
			{		

				while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

				IFS0bits.ADIF = 0;	//ClearFlag 
				VoltageMeas[AddPointer] = ADCBUF0;
				CurrentMeas[AddPointer] = ADCBUF1;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUF2;
				CurrentMeas[AddPointer] = ADCBUF3;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUF4;
				CurrentMeas[AddPointer] = ADCBUF5;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUF6;
				CurrentMeas[AddPointer] = ADCBUF7;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUF8;
				CurrentMeas[AddPointer] = ADCBUF9;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUFA;
				CurrentMeas[AddPointer] = ADCBUFB;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUFC;
				CurrentMeas[AddPointer] = ADCBUFD;
				AddPointer ++;
				VoltageMeas[AddPointer] = ADCBUFE;
				CurrentMeas[AddPointer] = ADCBUFF;
				AddPointer ++;

			}
		
			ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling
	
			//
			IIRCanonicInit (&IIR_BP_10KFilter); //Clear Filter Histroy
			AddPointer = 0;
			while (AddPointer < 20 ) //Run filter & ignore output in order to set up history values
			{
				IIRCanonic (_10K_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_10KFilter  );
				AddPointer ++;
			}
	
			//Fix Voltage Offset
			OffsetShift =( ( Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2))])  + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 1)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 2)])  + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+ 3)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+4)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+5)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+6)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+7)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+8)]) + Fract2Float(FilteredVoltageMeas[((_10K_NUM_SAMPLES/2)+9)])  ) / 5);
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _10K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}
			
			//
			IIRCanonicInit (&IIR_BP_10KFilter);
			AddPointer = 0;
			while (AddPointer < 20) // Run filter & ignore output in order to set up history values
			{
				IIRCanonic (_10K_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_10KFilter  );
				AddPointer ++;
			}
	
			//Fix Current Offset
			OffsetShift =( ( Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2))])  + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 1)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 2)])  + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+ 3)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+4)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+5)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+6)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+7)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+8)]) + Fract2Float(FilteredCurrentMeas[((_10K_NUM_SAMPLES/2)+9)])  ) / 5);
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _10K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				//Find Pre Zero Crossing
				ZeroCrossV = 39;  //Start at middle section
				ZeroCompare = Negative;
				
				//Find Positive Section First
				while ((ZeroCompare == Negative) && (ZeroCrossV < 60))
				{
					if ( Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCompare = Negative;
					}

					else
					{
						ZeroCompare = Positive;
					}
					ZeroCrossV++;
				}
				// Look For Negative Transision
				while (ZeroCrossV < 60)
				{
					if (Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCrossV --;
						break; //Found Trasision Point
					}

					else
					{
						ZeroCrossV++;
					}	
				}		
				
				CrossingGuess =  (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) / ( (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) - (Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)])));
		
				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/5) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/5) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );

				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetV = CrossingTemp;
		
				while (ZeroOffsetV < (CrossingTemp + 0.10))
				{
		
					CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetV)/5) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetV)/5) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );

					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetV;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetV = ZeroOffsetV + 0.001;
				
				}
		
				ZeroOffsetV = CrossingGuess;
		
				PeakV = fabs((Fract2Float(FilteredVoltageMeas[(ZeroCrossV - 2)])) /  (sinf ( ((3 - (ZeroOffsetV)) / 5) * PI )));
		
				//Scale Peak Value
				PeakV = PeakV / VGain;
				if (VGain >= 64)
				{
					PeakV = PeakV / 0.8334;
				}
		
				//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
				//Find Pre Zero Crossing
				ZeroCrossA = ZeroCrossV;  //Start From V point
			
				//Find Current Sign
				if (Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0)
				{
					ZeroCompare = Negative;
				}

				else
				{
					ZeroCompare = Positive;
				}
		
				// Look For Negative Transision
				while (ZeroCrossA < 60)
				{
					if ( ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0) && (ZeroCompare == Positive)) || ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) > 0) && (ZeroCompare == Negative))  )
					{
						ZeroCrossA --;
						break; //Found Trasision Point
					}

					else
					{
						ZeroCrossA++;
					}	
				}		
		
				if ( (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])) < 0 )
				{		
					CrossingGuess = ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}

				else
				{
					CrossingGuess = ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}
		
		
				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/5) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/5) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );

				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetA = CrossingTemp;
		
				while (ZeroOffsetA < (CrossingTemp + 0.10))
				{
		
							CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetA)/5) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetA)/5) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );
		
					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetA;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetA = ZeroOffsetA + 0.001;
				
				}
		
				ZeroOffsetA = CrossingGuess;
		
				//Peak Value Prior To Scaling
				PeakA =(fabs ((Fract2Float(FilteredCurrentMeas[(ZeroCrossA -2)])) / (sinf ( ((3 - (ZeroOffsetA)) / 5) * PI )))); 
		
				//Scale Peak Value
				PeakA = PeakA / AGain;
		
				if (AGain >= 64)
				{
					PeakA = PeakA / 0.8334;
				}
		
				Z10K = Z10K +  (PeakV / (PeakA / 1000));
		
				if (FilteredCurrentMeas[ZeroCrossA] > 0)
				{
					THETA10K = THETA10K + ( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV) )  / 10) * 360) - 180;
				}

				else
				{
					THETA10K = THETA10K + ( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV))  / 10) * 360);
				}

				AvCount ++;
		
			}

			Z10K = Z10K / Average;
			THETA10K = THETA10K / Average;


			//Fix Phase Shift From PGA Switch
			if (VGain >= 64)
			{
				THETA10K = THETA10K + 28;	
			}	
			//Check Case Of Maximum Impedance

			else if (AGain >= 64)
			{
				THETA10K = THETA10K - 28;	
			}	

		}

	//Reset DDS Frequency
	RESET_DDS_SPI ();

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

}


/////////////////////////////////////////////////////////////////


int TEST_1K(int Average)
{
	extern IIRCanonicStruct IIR_BP_1KFilter;
	extern FIRStruct V1KFIR_DEC_VFilter;
	extern FIRStruct A1KFIR_DEC_AFilter;

	//Local Variables
	float VHigh;
	float AHigh;
	int	VGain;
	int	AGain;
	int TEST_FINISH;
	int AddPointer;
	int BufPointer;
	int	AvCount;
	int	ZeroCrossV;
	int	ZeroCrossA;
	int ZeroCompare;
	float ZeroOffsetV;
	float ZeroOffsetA;
	float PeakV;
	float PeakA;
	float OffsetShift;
	float OffsetTemp;
	float CrossingError;
	float CrossingErrorMin;
	float CrossingGuess;	
	float CrossingTemp;

	//Initialize Locals	
	VHigh = 0;
	AHigh = 0;
	AGain = 1;
	VGain = 1;
	TEST_FINISH = 0;
	AddPointer = 0;

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

	//Set ADC 	
	set_ANALOG_LCR();

	//Set DDS Frequency
	SET_FREQ_DDS_SPI (F1KHz);
	RUN_DDS_SPI();
	GenDelay(50, UPFREQ);  // Settle DDS Output & Allow ADC Settle

	//Run Test
	while	(TEST_FINISH == 0)
	{
		GenDelay(25, UPFREQ);	//Allow New Gain Settle

		//Clear Filter Histroy
		FIRDelayInit ( &V1KFIR_DEC_VFilter);
		FIRDelayInit ( &A1KFIR_DEC_AFilter);

		AddPointer = 0; //Reset Pointer

		IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
		ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling

		BufPointer = 0;

		//Pre Read Voltage (8 Points) - Allow for interlaced deceminate
		while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

		IFS0bits.ADIF = 0;	//ClearFlag 
		FilteredVoltageMeas[BufPointer] = ADCBUF0;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF2;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF4;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF6;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF8;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFA;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFC;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFE;
		BufPointer ++;


		while (AddPointer < ((_1K_NUM_SAMPLES /2) + 50) )  //Collect Sample Set
		{		
			BufPointer = 0;

			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF0;
			FilteredCurrentMeas[BufPointer] = ADCBUF1;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF2;
			FilteredCurrentMeas[BufPointer] = ADCBUF3;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF4;
			FilteredCurrentMeas[BufPointer] = ADCBUF5;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer+ 8 ] = ADCBUF6;
			FilteredCurrentMeas[BufPointer] = ADCBUF7;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8 ] = ADCBUF8;
			FilteredCurrentMeas[BufPointer] = ADCBUF9;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer+ 8] = ADCBUFA;
			FilteredCurrentMeas[BufPointer] = ADCBUFB;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUFC;
			FilteredCurrentMeas[BufPointer] = ADCBUFD;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUFE;
			FilteredCurrentMeas[BufPointer] = ADCBUFF;
			BufPointer ++;

			//Deceminate V
			FIRDecimate (0x02,DecimateResult,FilteredVoltageMeas, &V1KFIR_DEC_VFilter ,8);  
			
			if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
			{
				VoltageMeas[((AddPointer - 50) * 2)] = DecimateResult [0];
				VoltageMeas[(((AddPointer - 50) * 2) + 1)] = DecimateResult [1];
			}


			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF0;
			FilteredCurrentMeas[BufPointer] = ADCBUF1;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF2;
			FilteredCurrentMeas[BufPointer] = ADCBUF3;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF4;
			FilteredCurrentMeas[BufPointer] = ADCBUF5;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF6;
			FilteredCurrentMeas[BufPointer] = ADCBUF7;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF8;
			FilteredCurrentMeas[BufPointer] = ADCBUF9;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFA;
			FilteredCurrentMeas[BufPointer] = ADCBUFB;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFC;
			FilteredCurrentMeas[BufPointer] = ADCBUFD;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFE;
			FilteredCurrentMeas[BufPointer] = ADCBUFF;

			//Deceminate V
			FIRDecimate (0x02,DecimateResult,FilteredCurrentMeas, &A1KFIR_DEC_AFilter ,8);  
			
			if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
			{
				CurrentMeas[((AddPointer - 50) * 2)] = DecimateResult [0];
				CurrentMeas[(((AddPointer - 50) * 2) + 1)] = DecimateResult [1];
			}

			AddPointer ++;

		}

		ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling

		//
		IIRCanonicInit (&IIR_BP_1KFilter); //Clear Filter Histroy
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (_1K_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_1KFilter);
			AddPointer ++;
		}

		//
		IIRCanonicInit (&IIR_BP_1KFilter);
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (_1K_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_1KFilter  );
			AddPointer ++;
		}

		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			//Fix Voltage Offset
			OffsetShift =( Fract2Float(FilteredVoltageMeas[0])  + Fract2Float(FilteredVoltageMeas[1]) + Fract2Float(FilteredVoltageMeas[2])  + Fract2Float(FilteredVoltageMeas[3]) + Fract2Float(FilteredVoltageMeas[4]) + Fract2Float(FilteredVoltageMeas[5]) + Fract2Float(FilteredVoltageMeas[6]) + Fract2Float(FilteredVoltageMeas[7]) + Fract2Float(FilteredVoltageMeas[8]) + Fract2Float(FilteredVoltageMeas[9]) + Fract2Float(FilteredVoltageMeas[10])  + Fract2Float(FilteredVoltageMeas[11]) + Fract2Float(FilteredVoltageMeas[12])  + Fract2Float(FilteredVoltageMeas[13]));
			OffsetShift =(( OffsetShift + Fract2Float(FilteredVoltageMeas[14])  + Fract2Float(FilteredVoltageMeas[15]) + Fract2Float(FilteredVoltageMeas[16])  + Fract2Float(FilteredVoltageMeas[17]) + Fract2Float(FilteredVoltageMeas[18]) + Fract2Float(FilteredVoltageMeas[19]) + Fract2Float(FilteredVoltageMeas[20]) + Fract2Float(FilteredVoltageMeas[21]) + Fract2Float(FilteredVoltageMeas[22]) + Fract2Float(FilteredVoltageMeas[23]) + Fract2Float(FilteredVoltageMeas[24])  + Fract2Float(FilteredVoltageMeas[25]) ) / 25)*2;
	
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;

			while (AddPointer < _1K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Voltage Gain
			AddPointer = 14; //Reset Pointer

			while (AddPointer < 40)
			{		
				if ((Fract2Float(FilteredVoltageMeas[AddPointer])) > VHigh)
				{
					VHigh = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			VGain = VGain * 2;

			if (VGain >= 64)
			{
				SW_V = 1;
				SET_SPI_PGA_GAIN((VGain/64), 'V');
			}

			else
			{
				SW_V = 0;
				SET_SPI_PGA_GAIN((VGain), 'V');
			}
		}

		if ((AHigh < GainLimit) && (AGain < 2048))
		{

			//Fix Current Offset
			OffsetShift =(  Fract2Float(FilteredCurrentMeas[0])  + Fract2Float(FilteredCurrentMeas[1]) + Fract2Float(FilteredCurrentMeas[2])  + Fract2Float(FilteredCurrentMeas[3]) + Fract2Float(FilteredCurrentMeas[4]) + Fract2Float(FilteredCurrentMeas[5]) + Fract2Float(FilteredCurrentMeas[6]) + Fract2Float(FilteredCurrentMeas[7]) + Fract2Float(FilteredCurrentMeas[8]) + Fract2Float(FilteredCurrentMeas[9]) + Fract2Float(FilteredCurrentMeas[10])  + Fract2Float(FilteredCurrentMeas[11]) + Fract2Float(FilteredCurrentMeas[12])  + Fract2Float(FilteredCurrentMeas[13]) );
			OffsetShift =(( OffsetShift + Fract2Float(FilteredCurrentMeas[14])  + Fract2Float(FilteredCurrentMeas[15]) + Fract2Float(FilteredCurrentMeas[16])  + Fract2Float(FilteredCurrentMeas[17]) + Fract2Float(FilteredCurrentMeas[18]) + Fract2Float(FilteredCurrentMeas[19]) + Fract2Float(FilteredCurrentMeas[20]) + Fract2Float(FilteredCurrentMeas[21]) + Fract2Float(FilteredCurrentMeas[22]) + Fract2Float(FilteredCurrentMeas[23]) + Fract2Float(FilteredCurrentMeas[24])  + Fract2Float(FilteredCurrentMeas[25]) ) / 25)*2;

			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;

			while (AddPointer < _1K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Current Gain
			AddPointer = 14; //Reset Pointer

			while (AddPointer < 40)
			{		
				if ( (Fract2Float(FilteredCurrentMeas[AddPointer])) > AHigh)
				{
					AHigh = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((AHigh < GainLimit) && (AGain < 2048))
		{
			AGain = AGain * 2;

			if (AGain >= 64)
			{
				SW_A = 1;
				SET_SPI_PGA_GAIN((AGain/64), 'A');
			}

			else
			{
				SW_A = 0;
				SET_SPI_PGA_GAIN((AGain), 'A');
			}
		}

		//Check if conditions are met to abort test	
		if (((VHigh >= GainLimit)&&(AHigh >= GainLimit)) || ((VGain == 2048)&&(VHigh < GainLimit)) || ((AGain == 2048)&&(AHigh < GainLimit)) )	
		{
			TEST_FINISH = 1; //Abort Test
		}

	}


	//Check Case Of Minimum Impedance
	if ((VGain == 2048)&&(VHigh < (GainLimit / 4)))
	{
		Z1K = 0;
		THETA1K = 0;	
	}	

	//Check Case Of Maximum Impedance
	else if ((AGain == 2048)&&(AHigh < (GainLimit / 4)))
	{
		Z1K = 999900 * 1000;
		THETA1K = 0;	
	}	
	// Impedance in range

	else 
	{
		THETA1K = 0;	
		Z1K = 0;
		AvCount = 0;

		while (AvCount < Average)
		{
			// Make Reading
		
			//Clear Filter Histroy
			FIRDelayInit ( &V1KFIR_DEC_VFilter);
			FIRDelayInit ( &A1KFIR_DEC_AFilter);
	
			AddPointer = 0; //Reset Pointer
	
			IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
			ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling
	
			BufPointer = 0;
			
			//Pre Read Voltage (8 Points) - Allow for interlaced deceminate
			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer] = ADCBUF0;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF2;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF4;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF6;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF8;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFA;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFC;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFE;
			BufPointer ++;
	
			while (AddPointer < ((_1K_NUM_SAMPLES /2) + 50) )  //Collect Sample Set
			{		
				BufPointer = 0;
	
				while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

				IFS0bits.ADIF = 0;	//ClearFlag 
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF0;
				FilteredCurrentMeas[BufPointer] = ADCBUF1;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF2;
				FilteredCurrentMeas[BufPointer] = ADCBUF3;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF4;
				FilteredCurrentMeas[BufPointer] = ADCBUF5;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer+ 8 ] = ADCBUF6;
				FilteredCurrentMeas[BufPointer] = ADCBUF7;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8 ] = ADCBUF8;
				FilteredCurrentMeas[BufPointer] = ADCBUF9;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer+ 8] = ADCBUFA;
				FilteredCurrentMeas[BufPointer] = ADCBUFB;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUFC;
				FilteredCurrentMeas[BufPointer] = ADCBUFD;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUFE;
				FilteredCurrentMeas[BufPointer] = ADCBUFF;
				BufPointer ++;
	
				//Deceminate V
				FIRDecimate (0x02,DecimateResult,FilteredVoltageMeas, &V1KFIR_DEC_VFilter ,8);  
				
				if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
				{
					VoltageMeas[((AddPointer - 50) * 2)] = DecimateResult [0];
					VoltageMeas[(((AddPointer - 50) * 2) + 1)] = DecimateResult [1];
				}
	
				while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

				IFS0bits.ADIF = 0;	//ClearFlag 
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF0;
				FilteredCurrentMeas[BufPointer] = ADCBUF1;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF2;
				FilteredCurrentMeas[BufPointer] = ADCBUF3;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF4;
				FilteredCurrentMeas[BufPointer] = ADCBUF5;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF6;
				FilteredCurrentMeas[BufPointer] = ADCBUF7;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF8;
				FilteredCurrentMeas[BufPointer] = ADCBUF9;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFA;
				FilteredCurrentMeas[BufPointer] = ADCBUFB;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFC;
				FilteredCurrentMeas[BufPointer] = ADCBUFD;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFE;
				FilteredCurrentMeas[BufPointer] = ADCBUFF;
	
				//Deceminate V
				FIRDecimate (0x02,DecimateResult,FilteredCurrentMeas, &A1KFIR_DEC_AFilter ,8);  
				
				if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
				{
					CurrentMeas[((AddPointer - 50) * 2)] = DecimateResult [0];
					CurrentMeas[(((AddPointer - 50) * 2) + 1)] = DecimateResult [1];
				}
	
				AddPointer ++;

			}

			ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling
	
			//
			IIRCanonicInit (&IIR_BP_1KFilter); //Clear Filter Histroy
			AddPointer = 0;

			while (AddPointer < 20 ) //Run filter & ignore output in order to set up history values
			{
				IIRCanonic (_1K_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_1KFilter  );
				AddPointer ++;
			}
	
			//Fix Voltage Offset
			OffsetShift =( Fract2Float(FilteredVoltageMeas[0])  + Fract2Float(FilteredVoltageMeas[1]) + Fract2Float(FilteredVoltageMeas[2])  + Fract2Float(FilteredVoltageMeas[3]) + Fract2Float(FilteredVoltageMeas[4]) + Fract2Float(FilteredVoltageMeas[5]) + Fract2Float(FilteredVoltageMeas[6]) + Fract2Float(FilteredVoltageMeas[7]) + Fract2Float(FilteredVoltageMeas[8]) + Fract2Float(FilteredVoltageMeas[9]) + Fract2Float(FilteredVoltageMeas[10])  + Fract2Float(FilteredVoltageMeas[11]) + Fract2Float(FilteredVoltageMeas[12])  + Fract2Float(FilteredVoltageMeas[13]));
			OffsetShift =( OffsetShift + Fract2Float(FilteredVoltageMeas[14])  + Fract2Float(FilteredVoltageMeas[15]) + Fract2Float(FilteredVoltageMeas[16])  + Fract2Float(FilteredVoltageMeas[17]) + Fract2Float(FilteredVoltageMeas[18]) + Fract2Float(FilteredVoltageMeas[19]) + Fract2Float(FilteredVoltageMeas[20]) + Fract2Float(FilteredVoltageMeas[21]) + Fract2Float(FilteredVoltageMeas[22]) + Fract2Float(FilteredVoltageMeas[23]) + Fract2Float(FilteredVoltageMeas[24]) ) / 12.5;
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;

			while (AddPointer < _1K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}
			
			//
			IIRCanonicInit (&IIR_BP_1KFilter);
			AddPointer = 0;

			while (AddPointer < 20) // Run filter & ignore output in order to set up history values
			{
				IIRCanonic (_1K_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_1KFilter  );
				AddPointer ++;
			}
	
			//Fix Current Offset
			OffsetShift =(  Fract2Float(FilteredCurrentMeas[0])  + Fract2Float(FilteredCurrentMeas[1]) + Fract2Float(FilteredCurrentMeas[2])  + Fract2Float(FilteredCurrentMeas[3]) + Fract2Float(FilteredCurrentMeas[4]) + Fract2Float(FilteredCurrentMeas[5]) + Fract2Float(FilteredCurrentMeas[6]) + Fract2Float(FilteredCurrentMeas[7]) + Fract2Float(FilteredCurrentMeas[8]) + Fract2Float(FilteredCurrentMeas[9]) + Fract2Float(FilteredCurrentMeas[10])  + Fract2Float(FilteredCurrentMeas[11]) + Fract2Float(FilteredCurrentMeas[12])  + Fract2Float(FilteredCurrentMeas[13]) );
			OffsetShift =( OffsetShift + Fract2Float(FilteredCurrentMeas[14])  + Fract2Float(FilteredCurrentMeas[15]) + Fract2Float(FilteredCurrentMeas[16])  + Fract2Float(FilteredCurrentMeas[17]) + Fract2Float(FilteredCurrentMeas[18]) + Fract2Float(FilteredCurrentMeas[19]) + Fract2Float(FilteredCurrentMeas[20]) + Fract2Float(FilteredCurrentMeas[21]) + Fract2Float(FilteredCurrentMeas[22]) + Fract2Float(FilteredCurrentMeas[23]) + Fract2Float(FilteredCurrentMeas[24]) ) / 12.5;
			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;

			while (AddPointer < _1K_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				//Find Pre Zero Crossing
				ZeroCrossV = 4;  //
				ZeroCompare = Negative;
				
				//Find Positive Section First
				while ((ZeroCompare == Negative) && (ZeroCrossV < 50))
				{
					if ( Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCompare = Negative;
					}

					else
					{
						ZeroCompare = Positive;
					}
					ZeroCrossV++;
				}
				// Look For Negative Transision
				while (ZeroCrossV < 50)
				{
					if (Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCrossV --;
						break; //Found Transition Point
					}

					else
					{
						ZeroCrossV++;
					}	
				}		

				CrossingGuess =  (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) / ( (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) - (Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)])));

				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/6.25) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/6.25) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );
	
				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetV = CrossingTemp;
		
				while (ZeroOffsetV < (CrossingTemp + 0.10))
				{
		
					CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetV)/6.25) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetV)/6.25) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );
		         // CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetA)/5) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetA)/5) * PI ))) )  -  ( (fabs(FilteredCurrentMeas[(ZeroCrossA)]))/(fabs(FilteredCurrentMeas[(ZeroCrossA + 1)])) ) );
		
					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetV;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetV = ZeroOffsetV + 0.001;
				
				}
		
				ZeroOffsetV = CrossingGuess;


	
				PeakV = fabs((Fract2Float(FilteredVoltageMeas[(ZeroCrossV - 3)])) /  (sinf ( ((3.25 - (ZeroOffsetV)) / 6.25) * PI )));
		
				//Scale Peak Value
				PeakV = PeakV / VGain;
				if (VGain >= 64)
				{
					PeakV = PeakV * 0.994;
				}
		
				//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
				//Find Pre Zero Crossing
				ZeroCrossA = ZeroCrossV;  //Start From V point
			
				//Find Current Sign
				if (Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0)
				{
					ZeroCompare = Negative;
				}
				else
				{
					ZeroCompare = Positive;
				}
		
				// Look For Negative Transision
				while (ZeroCrossA < 40)
				{
					if ( ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0) && (ZeroCompare == Positive)) || ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) > 0) && (ZeroCompare == Negative))  )
					{
						ZeroCrossA --;
						break; //Found Trasision Point
					}
					else
					{
						ZeroCrossA++;
					}	
				}		
		
				if ( (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])) < 0 )
				{		
					CrossingGuess = ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}

				else
				{
					CrossingGuess = ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}
		
		
				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/6.25) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/6.25) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );
		
				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetA = CrossingTemp;
		
				while (ZeroOffsetA < (CrossingTemp + 0.10))
				{
		
					CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetA)/6.25) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetA)/6.25) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );
		
					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetA;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetA = ZeroOffsetA + 0.001;
				
				}
		
				ZeroOffsetA = CrossingGuess;
		
				//Peak Value Prior To Scaling
				PeakA =(fabs ((Fract2Float(FilteredCurrentMeas[(ZeroCrossA -3)])) / (sinf ( ((3.25 - (ZeroOffsetA)) / 6.25) * PI )))); 
		
				//Scale Peak Value
				PeakA = PeakA / AGain;
		
				if (AGain >= 64)
				{
					PeakA = PeakA * 0.994;
				}
		
				Z1K =Z1K +  (PeakV / (PeakA / 1000));
		
				if (FilteredCurrentMeas[ZeroCrossA] > 0)
				{
					THETA1K = THETA1K + (( ( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV) )  / 12.5) * 360) - 180 ) + 28.8 );
				}

				else
				{
					THETA1K = THETA1K + (( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV))  / 12.5) * 360) + 28.8);
				}

				AvCount ++;
		
			}

			Z1K = Z1K / Average;
			THETA1K = THETA1K / Average;


			//Fix Phase Shift From PGA Switch
			if (VGain >= 64)
			{
				THETA1K = THETA1K + 3.3;	
			}	
			//Check Case Of Maximum Impedance
			else if (AGain >= 64)
			{
				THETA1K = THETA1K - 3.3;	
			}	

		}

	//Reset DDS Frequency
	RESET_DDS_SPI ();

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

}


/////////////////////////////////////////////////////////////////


int TEST_100(int Average)
{

	extern IIRCanonicStruct IIR_BP_100Filter;
	extern FIRStruct V100FIR_DEC_VFilter;
	extern FIRStruct A100FIR_DEC_AFilter;

	//Local Variables
	float VHigh;
	float AHigh;
	int	VGain;
	int	AGain;
	int TEST_FINISH;
	int AddPointer;
	int BufPointer;
	int	AvCount;
	int	ZeroCrossV;
	int	ZeroCrossA;
	int ZeroCompare;
	float ZeroOffsetV;
	float ZeroOffsetA;
	float PeakV;
	float PeakA;
	float OffsetShift;
	float OffsetTemp;
	float CrossingError;
	float CrossingErrorMin;
	float CrossingGuess;	
	float CrossingTemp;

	//Initialize Locals	
	VHigh = 0;
	AHigh = 0;
	AGain = 1;
	VGain = 1;
	TEST_FINISH = 0;
	AddPointer = 0;

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

	//Set ADC 	
	set_ANALOG_LCR();

	//Set DDS Frequency
	SET_FREQ_DDS_SPI (F100Hz);
	RUN_DDS_SPI();
	GenDelay(50, UPFREQ);  // Settle DDS Output & Allow ADC Settle

	//Run Test
	while	(TEST_FINISH == 0)
	{
		GenDelay(25, UPFREQ);	//Allow New Gain Settle

		//Clear Filter Histroy
		FIRDelayInit ( &V100FIR_DEC_VFilter);
		FIRDelayInit ( &A100FIR_DEC_AFilter);

		AddPointer = 0; //Reset Pointer

		IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
		ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling

		BufPointer = 0;

		//Pre Read Voltage (8 Points) - Allow for interlaced deceminate
		while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

		IFS0bits.ADIF = 0;	//ClearFlag 
		FilteredVoltageMeas[BufPointer] = ADCBUF0;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF2;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF4;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF6;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUF8;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFA;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFC;
		BufPointer ++;
		FilteredVoltageMeas[BufPointer] = ADCBUFE;
		BufPointer ++;


		while (AddPointer < ((_100_NUM_SAMPLES) + 50) )  //Collect Sample Set
		{		
			BufPointer = 0;

			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF0;
			FilteredCurrentMeas[BufPointer] = ADCBUF1;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF2;
			FilteredCurrentMeas[BufPointer] = ADCBUF3;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUF4;
			FilteredCurrentMeas[BufPointer] = ADCBUF5;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer+ 8 ] = ADCBUF6;
			FilteredCurrentMeas[BufPointer] = ADCBUF7;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8 ] = ADCBUF8;
			FilteredCurrentMeas[BufPointer] = ADCBUF9;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer+ 8] = ADCBUFA;
			FilteredCurrentMeas[BufPointer] = ADCBUFB;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUFC;
			FilteredCurrentMeas[BufPointer] = ADCBUFD;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer + 8] = ADCBUFE;
			FilteredCurrentMeas[BufPointer] = ADCBUFF;
			BufPointer ++;

			//Deceminate V
			FIRDecimate (0x01,DecimateResult,FilteredVoltageMeas, &V100FIR_DEC_VFilter ,16);  
			
			if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
			{
				VoltageMeas[(AddPointer - 50)] = DecimateResult [0];
			}


			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF0;
			FilteredCurrentMeas[BufPointer] = ADCBUF1;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF2;
			FilteredCurrentMeas[BufPointer] = ADCBUF3;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF4;
			FilteredCurrentMeas[BufPointer] = ADCBUF5;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF6;
			FilteredCurrentMeas[BufPointer] = ADCBUF7;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUF8;
			FilteredCurrentMeas[BufPointer] = ADCBUF9;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFA;
			FilteredCurrentMeas[BufPointer] = ADCBUFB;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFC;
			FilteredCurrentMeas[BufPointer] = ADCBUFD;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer - 8] = ADCBUFE;
			FilteredCurrentMeas[BufPointer] = ADCBUFF;

			//Deceminate V
			FIRDecimate (0x01,DecimateResult,FilteredCurrentMeas, &A100FIR_DEC_AFilter ,16);  
			
			if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
			{
				CurrentMeas[(AddPointer - 50)] = DecimateResult [0];
			}

			AddPointer ++;

		}

		ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling

		//
		IIRCanonicInit (&IIR_BP_100Filter); //Clear Filter Histroy
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (62,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_100Filter);
			AddPointer ++;
		}

		IIRCanonic (_100_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_100Filter);
		//
		IIRCanonicInit (&IIR_BP_100Filter);
		AddPointer = 0;
		while (AddPointer < 20) //Run filter & ignore output in order to set up history values
		{
			IIRCanonic (62,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_100Filter  );
			AddPointer ++;
		}

		IIRCanonic (_100_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_100Filter  );

		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			//Fix Voltage Offset
			OffsetShift =( Fract2Float(FilteredVoltageMeas[0]) + Fract2Float(FilteredVoltageMeas[1]) + Fract2Float(FilteredVoltageMeas[2]) + Fract2Float(FilteredVoltageMeas[3]) + Fract2Float(FilteredVoltageMeas[4]) + Fract2Float(FilteredVoltageMeas[5]) +  Fract2Float(FilteredVoltageMeas[6]) + Fract2Float(FilteredVoltageMeas[7]) + Fract2Float(FilteredVoltageMeas[8]) + Fract2Float(FilteredVoltageMeas[9]) + Fract2Float(FilteredVoltageMeas[31]) + Fract2Float(FilteredVoltageMeas[32]) + Fract2Float(FilteredVoltageMeas[33]) + Fract2Float(FilteredVoltageMeas[34]) + Fract2Float(FilteredVoltageMeas[35]) + Fract2Float(FilteredVoltageMeas[36]) +  Fract2Float(FilteredVoltageMeas[37]) + Fract2Float(FilteredVoltageMeas[38]) + Fract2Float(FilteredVoltageMeas[39]) + Fract2Float(FilteredVoltageMeas[40]) ) / 10; 

			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _100_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Voltage Gain
			AddPointer = 4; //Reset Pointer
			while (AddPointer < 68)
			{		
				if ((Fract2Float(FilteredVoltageMeas[AddPointer])) > VHigh)
				{
					VHigh = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((VHigh < GainLimit) && (VGain < 2048))
		{
			VGain = VGain * 2;

			if (VGain >= 64)
			{
				SW_V = 1;
				SET_SPI_PGA_GAIN((VGain/64), 'V');
			}

			else
			{
				SW_V = 0;
				SET_SPI_PGA_GAIN((VGain), 'V');
			}
		}

		if ((AHigh < GainLimit) && (AGain < 2048))
		{
			//Fix Current Offset
			OffsetShift =( Fract2Float(FilteredCurrentMeas[0]) + Fract2Float(FilteredCurrentMeas[1]) + Fract2Float(FilteredCurrentMeas[2]) + Fract2Float(FilteredCurrentMeas[3]) + Fract2Float(FilteredCurrentMeas[4]) + Fract2Float(FilteredCurrentMeas[5]) +  Fract2Float(FilteredCurrentMeas[6]) + Fract2Float(FilteredCurrentMeas[7]) + Fract2Float(FilteredCurrentMeas[8]) + Fract2Float(FilteredCurrentMeas[9]) + Fract2Float(FilteredCurrentMeas[31]) + Fract2Float(FilteredCurrentMeas[32]) + Fract2Float(FilteredCurrentMeas[33]) + Fract2Float(FilteredCurrentMeas[34]) + Fract2Float(FilteredCurrentMeas[35]) + Fract2Float(FilteredCurrentMeas[36]) +  Fract2Float(FilteredCurrentMeas[37]) + Fract2Float(FilteredCurrentMeas[38]) + Fract2Float(FilteredCurrentMeas[39]) + Fract2Float(FilteredCurrentMeas[40]) ) / 10; 

			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _100_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//Check Current Gain
			AddPointer = 4; //Reset Pointer
			while (AddPointer < 68)
			{		
				if ( (Fract2Float(FilteredCurrentMeas[AddPointer])) > AHigh)
				{
					AHigh = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				}	
				AddPointer++;
			}
		}

		// Set New Gain If Required
		if ((AHigh < GainLimit) && (AGain < 2048))
		{
			AGain = AGain * 2;

			if (AGain >= 64)
			{
				SW_A = 1;
				SET_SPI_PGA_GAIN((AGain/64), 'A');
			}

			else
			{
				SW_A = 0;
				SET_SPI_PGA_GAIN((AGain), 'A');
			}
		}

		//Check if conditions are met to abort test	
		if (((VHigh >= GainLimit)&&(AHigh >= GainLimit)) || ((VGain == 2048)&&(VHigh < GainLimit)) || ((AGain == 2048)&&(AHigh < GainLimit)) )	
		{
			TEST_FINISH = 1; //Abort Test
		}

	}


	//Check Case Of Minimum Impedance
	if ((VGain == 2048)&&(VHigh < (GainLimit / 4)))
	{
		Z100 = 0;
		THETA100 = 0;	
	}	

	//Check Case Of Maximum Impedance
	else if ((AGain == 2048)&&(AHigh < (GainLimit / 4)))
	{
		Z100 = 999900 * 1000;
		THETA100 = 0;	
	}	

	// Impedance in range
	else 
	{
		THETA100 = 0;	
		Z100 = 0;
		AvCount = 0;

		while (AvCount < Average)
		{
			// Make Reading
			
			//Clear Filter Histroy
			FIRDelayInit ( &V100FIR_DEC_VFilter);
			FIRDelayInit ( &A100FIR_DEC_AFilter);
	
			AddPointer = 0; //Reset Pointer
	
			IFS0bits.ADIF = 0;	//Clear ADC Interupt Flag 
			ADCON1bits.ASAM = 1;   //Start Automatic ADC Sampling
	
			BufPointer = 0;
			
			//Pre Read Voltage (8 Points) - Allow for interlaced deceminate
			while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

			IFS0bits.ADIF = 0;	//ClearFlag 
			FilteredVoltageMeas[BufPointer] = ADCBUF0;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF2;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF4;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF6;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUF8;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFA;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFC;
			BufPointer ++;
			FilteredVoltageMeas[BufPointer] = ADCBUFE;
			BufPointer ++;
	
			while (AddPointer < ((_100_NUM_SAMPLES) + 50) )  //Collect Sample Set
			{		
				BufPointer = 0;
	
				while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

				IFS0bits.ADIF = 0;	//ClearFlag 
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF0;
				FilteredCurrentMeas[BufPointer] = ADCBUF1;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF2;
				FilteredCurrentMeas[BufPointer] = ADCBUF3;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUF4;
				FilteredCurrentMeas[BufPointer] = ADCBUF5;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer+ 8 ] = ADCBUF6;
				FilteredCurrentMeas[BufPointer] = ADCBUF7;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8 ] = ADCBUF8;
				FilteredCurrentMeas[BufPointer] = ADCBUF9;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer+ 8] = ADCBUFA;
				FilteredCurrentMeas[BufPointer] = ADCBUFB;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUFC;
				FilteredCurrentMeas[BufPointer] = ADCBUFD;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer + 8] = ADCBUFE;
				FilteredCurrentMeas[BufPointer] = ADCBUFF;
				BufPointer ++;
	
				//Deceminate V
				FIRDecimate (0x01,DecimateResult,FilteredVoltageMeas, &V100FIR_DEC_VFilter ,16);  
				
				if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
				{
					VoltageMeas[(AddPointer - 50)] = DecimateResult [0];
				}
	
	
				while (IFS0bits.ADIF == 0); //Wait For Interupt Flag

				IFS0bits.ADIF = 0;	//ClearFlag 
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF0;
				FilteredCurrentMeas[BufPointer] = ADCBUF1;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF2;
				FilteredCurrentMeas[BufPointer] = ADCBUF3;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF4;
				FilteredCurrentMeas[BufPointer] = ADCBUF5;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF6;
				FilteredCurrentMeas[BufPointer] = ADCBUF7;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUF8;
				FilteredCurrentMeas[BufPointer] = ADCBUF9;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFA;
				FilteredCurrentMeas[BufPointer] = ADCBUFB;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFC;
				FilteredCurrentMeas[BufPointer] = ADCBUFD;
				BufPointer ++;
				FilteredVoltageMeas[BufPointer - 8] = ADCBUFE;
				FilteredCurrentMeas[BufPointer] = ADCBUFF;
	
				//Deceminate V
				FIRDecimate (0x01,DecimateResult,FilteredCurrentMeas, &A100FIR_DEC_AFilter ,16);  
				
				if (AddPointer >= 50) //Use First 50 Runs to Set Up Filter History
				{
					CurrentMeas[(AddPointer - 50)] = DecimateResult [0];
				}
	
				AddPointer ++;

			}

			ADCON1bits.ASAM = 0;   //Stop Automatic ADC Sampling
	
			//
			IIRCanonicInit (&IIR_BP_100Filter); //Clear Filter Histroy
			AddPointer = 0;
			while (AddPointer < 50 ) //Run filter & ignore output in order to set up history values
			{
				IIRCanonic (62,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_100Filter  );
				AddPointer ++;
			}
				IIRCanonic (_100_NUM_SAMPLES,FilteredVoltageMeas, VoltageMeas ,&IIR_BP_100Filter  );

			//Fix Voltage Offset
			OffsetShift =( Fract2Float(FilteredVoltageMeas[0]) + Fract2Float(FilteredVoltageMeas[1]) + Fract2Float(FilteredVoltageMeas[2]) + Fract2Float(FilteredVoltageMeas[3]) + Fract2Float(FilteredVoltageMeas[4]) + Fract2Float(FilteredVoltageMeas[5]) +  Fract2Float(FilteredVoltageMeas[6]) + Fract2Float(FilteredVoltageMeas[7]) + Fract2Float(FilteredVoltageMeas[8]) + Fract2Float(FilteredVoltageMeas[9]) + Fract2Float(FilteredVoltageMeas[31]) + Fract2Float(FilteredVoltageMeas[32]) + Fract2Float(FilteredVoltageMeas[33]) + Fract2Float(FilteredVoltageMeas[34]) + Fract2Float(FilteredVoltageMeas[35]) + Fract2Float(FilteredVoltageMeas[36]) +  Fract2Float(FilteredVoltageMeas[37]) + Fract2Float(FilteredVoltageMeas[38]) + Fract2Float(FilteredVoltageMeas[39]) + Fract2Float(FilteredVoltageMeas[40]) ) / 10; 

			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _100_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredVoltageMeas[AddPointer]));
				FilteredVoltageMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}
			
			//
			IIRCanonicInit (&IIR_BP_100Filter);
			AddPointer = 0;
			while (AddPointer < 50) // Run filter & ignore output in order to set up history values
			{
				IIRCanonic (62,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_100Filter  );
				AddPointer ++;
			}
			IIRCanonic (_100_NUM_SAMPLES,FilteredCurrentMeas, CurrentMeas ,&IIR_BP_100Filter  );

			//Fix Current Offset
			OffsetShift =( Fract2Float(FilteredCurrentMeas[0]) + Fract2Float(FilteredCurrentMeas[1]) + Fract2Float(FilteredCurrentMeas[2]) + Fract2Float(FilteredCurrentMeas[3]) + Fract2Float(FilteredCurrentMeas[4]) + Fract2Float(FilteredCurrentMeas[5]) +  Fract2Float(FilteredCurrentMeas[6]) + Fract2Float(FilteredCurrentMeas[7]) + Fract2Float(FilteredCurrentMeas[8]) + Fract2Float(FilteredCurrentMeas[9]) + Fract2Float(FilteredCurrentMeas[31]) + Fract2Float(FilteredCurrentMeas[32]) + Fract2Float(FilteredCurrentMeas[33]) + Fract2Float(FilteredCurrentMeas[34]) + Fract2Float(FilteredCurrentMeas[35]) + Fract2Float(FilteredCurrentMeas[36]) +  Fract2Float(FilteredCurrentMeas[37]) + Fract2Float(FilteredCurrentMeas[38]) + Fract2Float(FilteredCurrentMeas[39]) + Fract2Float(FilteredCurrentMeas[40]) ) / 10; 

			//Check Offset Over One Cycle + Shift Data Set
			AddPointer = 0;
			while (AddPointer < _100_NUM_SAMPLES ) //Run filter & ignore output in order to set up history values
			{
				OffsetTemp = (Fract2Float(FilteredCurrentMeas[AddPointer]));
				FilteredCurrentMeas[AddPointer] = Float2Fract(OffsetTemp - OffsetShift);
				AddPointer ++;
			}

			//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				//Find Pre Zero Crossing
				ZeroCrossV = 2;  //
				ZeroCompare = Negative;
				
				//Find Positive Section First
				while ((ZeroCompare == Negative) && (ZeroCrossV < 80))
				{
					if ( Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCompare = Negative;
					}

					else
					{
						ZeroCompare = Positive;
					}
					ZeroCrossV++;
				}
				// Look For Negative Transision
				while (ZeroCrossV < 80)
				{
					if (Fract2Float(FilteredVoltageMeas[ZeroCrossV]) < 0)
					{
						ZeroCrossV --;
						break; //Found Transition Point
					}

					else
					{
						ZeroCrossV++;
					}	
				}		

				CrossingGuess =  (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) / ( (Fract2Float(FilteredVoltageMeas[(ZeroCrossV)])) - (Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)])));

				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/31) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/31) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );
	
				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetV = CrossingTemp;
		
				while (ZeroOffsetV < (CrossingTemp + 0.10))
				{
		
					CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetV)/31) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetV)/31) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV)]) ))/(fabs(Fract2Float(FilteredVoltageMeas[(ZeroCrossV + 1)]))) ) );

					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetV;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetV = ZeroOffsetV + 0.001;
				
				}
		
				ZeroOffsetV = CrossingGuess;


	
				PeakV = fabs((Fract2Float(FilteredVoltageMeas[(ZeroCrossV - 2)])) /  (sinf ( ((29 - (ZeroOffsetV)) / 31) * PI )));
		
				//Scale Peak Value
				PeakV = PeakV / VGain;
				if (VGain >= 64)
				{
					//
				}
		
				//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
				//Find Pre Zero Crossing
				ZeroCrossA = ZeroCrossV;  //Start From V point
			
				//Find Current Sign
				if (Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0)
				{
					ZeroCompare = Negative;
				}
				else
				{
					ZeroCompare = Positive;
				}
		
				// Look For Negative Transision
				while (ZeroCrossA < 94)
				{
					if ( ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) < 0) && (ZeroCompare == Positive)) || ((Fract2Float (FilteredCurrentMeas[ZeroCrossA]) > 0) && (ZeroCompare == Negative))  )
					{
						ZeroCrossA --;
						break; //Found Trasision Point
					}
					else
					{
						ZeroCrossA++;
					}	
				}		
		
				if ( (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])) < 0 )
				{		
					CrossingGuess = ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (-1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}
				else
				{
					CrossingGuess = ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  )  /  ((1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA)])))  - (1 * (Fract2Float(FilteredCurrentMeas[(ZeroCrossA +1)]))) );
				}
		
		
				CrossingErrorMin = fabs (  ( ( fabs(sinf( ((CrossingGuess)/31) * PI ))) / (fabs(sinf( ((1 - CrossingGuess)/31) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );
		
				if (CrossingGuess >= 0.95)
				{
					CrossingTemp = 0.90;
				} 

				else if (CrossingGuess <= 0.05)
				{
					CrossingTemp = 0.00;
				}

				else
				{
					CrossingTemp = CrossingGuess - 0.05;
				}
			
				ZeroOffsetA = CrossingTemp;
		
				while (ZeroOffsetA < (CrossingTemp + 0.10))
				{
		
					CrossingError = fabs (  ( ( fabs(sinf( ((ZeroOffsetA)/31) * PI ))) / (fabs(sinf( ((1 - ZeroOffsetA)/31) * PI ))) )  -  ( ( fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA)]) ))/(fabs(Fract2Float(FilteredCurrentMeas[(ZeroCrossA + 1)]))) ) );
		
					if ( CrossingError < CrossingErrorMin)
					{
						CrossingGuess = ZeroOffsetA;
						CrossingErrorMin = CrossingError;
					}
		
					ZeroOffsetA = ZeroOffsetA + 0.001;
				
				}
		
				ZeroOffsetA = CrossingGuess;
		
				//Peak Value Prior To Scaling
				PeakA =(fabs ((Fract2Float(FilteredCurrentMeas[(ZeroCrossA -2)])) / (sinf ( ((29 - (ZeroOffsetA)) / 31) * PI )))); 
		
				//Scale Peak Value
				PeakA = PeakA / AGain;
		
				if (AGain >= 64)
				{
					//	
				}
		
				Z100 =Z100 +  (PeakV / (PeakA / 1000));
		
				if (FilteredCurrentMeas[ZeroCrossA] > 0)
				{
					THETA100 = THETA100 + (( ( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV) )  / 62) * 360) - 180 ) + 2.90322586 );
				}

				else
				{
					THETA100 = THETA100 + (( (( (ZeroCrossA - ZeroCrossV) + (ZeroOffsetA - ZeroOffsetV))  / 62) * 360) + 2.90322586);
				}

				AvCount ++;
		
			}

			Z100 = Z100 / Average;
			THETA100 = THETA100 / Average;


			//Fix Phase Shift From PGA Switch
			if (VGain >= 64)
			{
				//	
			}	
			//Check Case Of Maximum Impedance
			else if (AGain >= 64)
			{
				//		
			}	

		}

	//Reset DDS Frequency
	RESET_DDS_SPI ();

	//Set Gain To Minimum	
	SW_V = 0;
	SET_SPI_PGA_GAIN(1, 'V');
	SW_A = 0;
	SET_SPI_PGA_GAIN(1, 'A');

}


/********************************************************************/


