/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        LCR METER INIT.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* Initialisation File For DSP LCR METER
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include <dsp.h>
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"

/*****************************************************************************/

void init_SPI()
{

	SPI1CONbits.FRMEN = 0; 		//FRAMED SUPPORT DISABLED
	SPI1CONbits.SPIFSD = 0;		//MASTER MODE (FRAMED SYNC DIR)
	SPI1CONbits.DISSDO = 0;		//SDO PIN UNDER MODULE CONTROL
	SPI1CONbits.MODE16 = 0;		//8 BIT WORD WIDTH
	SPI1CONbits.SMP = 1;		//SAMPLE INPUT DATA AT END OF FRAME
	SPI1CONbits.SSEN = 0;		//PIN NOT IN USE
	SPI1CONbits.MSTEN = 1;		//ENABLE MASTER MODE
	//Initialy Set SPI For LCD
	SPI1CONbits.SPRE = 7;  		//SECONDARY PRESCALE = 1:1
	SPI1CONbits.PPRE = 1;  		//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKP = 1;		//CLOCK IDLE LOW
	SPI1CONbits.CKE = 0;		//CLOCK ON IDLE->ACTIVE
	
	SPI1STATbits.SPIROV = 0;	//NO OVERFLOW
	SPI1STATbits.SPIEN = 1;		//ENABLE SPI MODULE
	SPI1STATbits.SPISIDL = 1; 	//STOP IN IDLE MODE
	
	TRISEbits.TRISE5 = 0;		//MAKE OUTPUT (GPIO CS)
	LATEbits.LATE5 = 1;			//SET HIGH

	TRISDbits.TRISD0 = 0;		//MAKE OUTPUT (DDS CS)
	LATDbits.LATD0 = 1;			//SET HIGH

	TRISDbits.TRISD1 = 0;		//MAKE OUTPUT (PGA Volts)
	LATDbits.LATD1 = 1;		//SET HIGH

	TRISBbits.TRISB5 = 0;		//MAKE OUTPUT (PGA Amps)
	LATBbits.LATB5 = 1; 		//SET HIGH

}


////////////////////////////////////////


void init_PWM()
{

	// Code Lifted From Microchip Example - Modified To Suit
    /* ~~~~~~~~~~~~~~~~~~~~~~ PWM1 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    /* PWM1 I/O Control Register register */

    PWMCON1bits.PEN1H      = 0;   /* PWM1H is not controlled by PWM module */
    PWMCON1bits.PEN1L      = 1;   /* PWM1L is controlled by PWM module */
    PWMCON1bits.PMOD1      = 1;   /* Select Indipendant Output PWM mode */

    /* Load PDC1 register with initial Duty Cycle value */

    PDC1 = 40;                    /* Set Initial Contrast */

    /* Load PDTR1 and ALTDTR1 register with preset dead time value */

    DTCON1bits.DTA    = 0;        /* Deadtime setting */
			                      /* Deadtime = 0sec */
    DTCON1bits.DTAPS = 0;         /* Deadtime setting */
                                  /* Deadtime = 0sec */

    /* ~~~~~~~~~~~~~~~~~~~~~~ PWM2 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    /* PWM2 I/O Control Register register */

    PWMCON1bits.PEN2H      = 0;   /* PWM2H is not controlled by PWM module */
    PWMCON1bits.PEN2L      = 1;   /* PWM2L is controlled by PWM module */
    PWMCON1bits.PMOD2      = 1;   /* Select Indipendant Output PWM mode */

 	/* Load PDC2 register with initial Duty Cycle value */

    PDC2 = 24;                    /* Set 50% For DDS MCLK  */

    /* ~~~~~~~~~~~~~~~~~~~~~~ PWM3 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    /* PWM2 I/O Control Register register */

    PWMCON1bits.PEN3H      = 0;   /* PWM3H is not controlled by PWM module */
    PWMCON1bits.PEN3L      = 0;   /* PWM3L is not controlled by PWM module */
    PWMCON1bits.PMOD3      = 1;   /* Select Indipendant Output PWM mode */

    /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
    /* Configure PTPER register to produce 100kHz PWM frequency */

    PTPER = 23;                   /* == 100kHz */
                                  /* PWM Config Register Clear */
	PTCON = 0; 					  /*Clear Reg */
    							  /* PWM Time Base Control Register */
 	PTCONbits.PTEN = 1;     	  /* Enable the PWM Module */

}


/////////////////////////////////////////////////////////////////


void init_LCD_PINS()
{

	TRISEbits.TRISE1 = 0;		//MAKE OUTPUT (LCD WR)
	LATEbits.LATE1 = 1;			//SET HIGH

	TRISEbits.TRISE3 = 0;		//MAKE OUTPUT (LCD CD) 
	LATEbits.LATE3 = 1;			//SET HIGH

	TRISEbits.TRISE4 = 0;		//MAKE OUTPUT (LCD CE)
	LATEbits.LATE4 = 1;			//SET HIGH

}


/////////////////////////////////////////////////////////////////


void init_PGA_SW_PINS()
{

	TRISCbits.TRISC13 = 0;		//MAKE OUTPUT (PGA SW AMPS)
	LATCbits.LATC13 = 0;		//SET LOW

	TRISCbits.TRISC14 = 0;		//MAKE OUTPUT (PGA SW VOLTS)
	LATCbits.LATC14 = 0;		//SET LOW

}


/////////////////////////////////////////////////////////////////


void init_ANALOG()
{

	ADCON1bits.ADON = 0x00;	//Ensure ADC is off before register writes.
	//

	ADPCFG = 0xFFE0;			//Make analog pins digital 
	TRISBbits.TRISB0 = 1;		//MAKE ANALOG PIN INPUT
	TRISBbits.TRISB1 = 1;		//MAKE ANALOG PIN INPUT
	TRISBbits.TRISB2 = 1;		//MAKE ANALOG PIN INPUT
	TRISBbits.TRISB3 = 1;		//MAKE ANALOG PIN INPUT
	TRISBbits.TRISB4 = 1;		//MAKE ANALOG PIN INPUT

	ADCON1 = 0X00;				//Clear Register
	//

	ADCON2 = 0X00;				//Clear Register
	//
	ADCON2bits.VCFG = 0X03;		//Vref+  & Vref- from external pins.
	ADCON2bits.BUFM = 0X00;		//One 16 word Buffer	

	ADCON3 = 0X00;				//Clear Register

	//
	ADCON3bits.ADRC = 0X00;		//Use internal clock source

	ADCHS = 0X00;				//Clear Register
	//

}


/////////////////////////////////////////////////////////////////


void set_ANALOG_BUTTON()
{

	ADCON1bits.ADON = 0x00;		//Ensure ADC is off before register writes.
	//

	ADCON1bits.FORM = 0X00;		//Use Integer Format
	ADCON1bits.SAMP = 0x00;
	ADCON1bits.SIMSAM = 0x00;	//Sequentual Sample Mode
	ADCON1bits.SSRC	= 0X07;		//Clocked sample time

	ADCON2bits.SMPI = 0X00;		//Interupt Every Conversion
	ADCON2bits.CHPS = 0X00;		//Converts CH0

	ADCON3bits.ADCS = 0X0F;		//TAD = 8 TCY 
	ADCON3bits.SAMC = 0X18;		//31TAD 

	ADCHSbits.CH123NA = 0X00;	//Set MUX A (BUTTON INPUTS)
	ADCHSbits.CH123SA = 0X01;
	ADCHSbits.CH0NA = 0X00;
	ADCHSbits.CH0SA	= 0X04;

	//
	ADCON1bits.ADON = 0x01;		//Enable ADC.

}


/////////////////////////////////////////////////////////////////


void set_ANALOG_OFFSET()
{

	ADCON1bits.ADON = 0x00;	//Ensure ADC is off before register writes.
	//

	ADCON1bits.FORM = 0X03;		//Use SIGNED FRACTIONAL Format
	ADCON1bits.SAMP = 0x00;
	ADCON1bits.SIMSAM = 0x00;	//Sequentual Sample Mode
	ADCON1bits.SSRC	= 0X07;		//Clocked sample time

	ADCON2bits.SMPI = 0X00;		//Interupt Every Conversion
	ADCON2bits.CHPS = 0X00;		//Converts CH0

	ADCON3bits.ADCS = 0X1F;		//TAD = 16 TCY 
	ADCON3bits.SAMC = 0X10;		//16TAD 

	ADCHSbits.CH123NA = 0X00;	//Set MUX A (LCR INPUTS)
	ADCHSbits.CH123SA = 0X01;
	ADCHSbits.CH0NA = 0X00;
	ADCHSbits.CH0SA	= 0X02; 	//Read Voltage Channel

	//
	ADCON1bits.ADON = 0x01;		//Enable ADC.

}


/////////////////////////////////////////////////////////////////


void set_ANALOG_LCR()
{

	ADCON1bits.ADON = 0x00;		//Ensure ADC is off before register writes.
	//

	ADCON1bits.FORM = 0X03;		//Use SIGNED FRACTIONAL Format
	ADCON1bits.SAMP = 0x00;
	ADCON1bits.SIMSAM = 0x01;	//Simultaneous Sample Mode
	ADCON1bits.SSRC	= 0X07;		//Clocked sample time

	ADCON2bits.SMPI = 0X0F;		//Interupt Every 16 conversions
	ADCON2bits.CHPS = 0X01;		//Converts CH0 & CH1

	ADCON3bits.ADCS = 0X0F;		//TAD = 30 TCY [200KS/s  (TSAMP = 3TAD)]
	ADCON3bits.SAMC = 0X06;		//6TAD (200KS/s)

	ADCHSbits.CH123NA = 0X00;	//Set MUX A (LCR INPUTS)
	ADCHSbits.CH123SA = 0X01;
	ADCHSbits.CH0NA = 0X00;
	ADCHSbits.CH0SA	= 0X02;

	//
	ADCON1bits.ADON = 0x01;		//Enable ADC.

}


/*****************************************************************************/

