/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        Main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* 
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include "dsp.h"
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"
	#include "Graphic LCD.h"
	#include "SPI GPIO.h"
	#include "SPI DDS.h"
	#include "SPI MCP PGA.h"
	#include "SPI MCP POT.h"
	#include "LCR METER INIT.h"
	#include "LCR METER FUNCS.h"

/********************************************************************/
//PROCESSOR SETTINGS

/* Device configuration register macros for building the hex file */
_FOSC(CSW_FSCM_OFF & XT_PLL16);          /* XT with 16xPLL oscillator, Failsafe clock off */
_FWDT(WDT_OFF);                         /* Watchdog timer disabled */
_FBORPOR(PBOR_ON & MCLR_EN);           /* Brown-out reset enabled, MCLR reset enabled */
_FGS(CODE_PROT_OFF);                    /* Code protect disabled */

/********************************************************************/
//DEFINES

	#define XTFREQ      	6000000         		//On-board Crystal frequency
	#define PLLMODE     	16               		//On-chip PLL setting
	#define FCY         	XTFREQ*PLLMODE/4        //Instruction Cycle Frequency
	/////

	#define	F10KHz			0x03	
	#define	F1KHz			0x02
	#define	F100Hz			0x01
	#define	AUTOHz			0x00

	#define	PARALLEL_MODE	0x02
	#define	SERIES_MODE 	0x01
	#define	AUTOH_MODE		0x00

	#define	SCREEN_BLANK 	0x00
	#define	SCREEN_RESULT 	0x01

	#define	DASH_SYM		0x00
	#define	RESISTOR_SYM 	0x01
	#define	CAPACITOR_SYM 	0x02
	#define	INDUCTOR_SYM	0x03


/*****************************************************************************/

int main( void )
{

	//Local Variables
	int BUTTON_NEW;
	int BUTTON_OLD;
	
	//Initialise Components
	init_ANALOG();
	init_SPI();
	init_PGA_SW_PINS();
	init_LCD_PINS();
	GenDelay(10, FCY);
	init_PWM();
	RESET_DDS_SPI();
	InitLCD();
	init_ANALOG();
	INIT_LCR_FUNCS();
	SET_SPI_MCP_POT(128);
	
	while (1) //Main Loop
	{
		//	Check For Button Push
		while (1)	//Wait for button release
		{
			if (GET_BUTTON_VAL() == 0)
			{
				GenDelay (50, FCY);		//DEBOUCE
				if (GET_BUTTON_VAL() == 0)
				{
					break;
				}
			}
		}
		BUTTON_OLD = 0;
	
		while (1) 	//Look for valid button press & debounce
		{
			BUTTON_NEW = (GET_BUTTON_VAL());
			if ((BUTTON_NEW != BUTTON_OLD) && (BUTTON_NEW != 0)) //Button state change
			{
				GenDelay (100, FCY);		//DEBOUCE
				BUTTON_NEW = (GET_BUTTON_VAL());
				BUTTON_OLD = BUTTON_NEW;
				GenDelay (50, FCY);			//DEBOUCE
				BUTTON_NEW = (GET_BUTTON_VAL());
				
				if (BUTTON_NEW == 4)
				{
					BUTTON_OLD = BUTTON_NEW;
					GenDelay (50, FCY);		//DEBOUCE
					BUTTON_NEW = (GET_BUTTON_VAL());
				}
		
				if ((BUTTON_NEW == BUTTON_OLD) && (BUTTON_NEW != 0))
				{
					break;	//EXIT LOOP
				}
			}	
			BUTTON_OLD = BUTTON_NEW;
		}
	
		if (BUTTON_NEW == 1) //Mode Button
		{
			CHANGE_MODE();
		}

		else if (BUTTON_NEW == 2) //Frequency Button
		{
			CHANGE_FREQ();
		}

		else if (BUTTON_NEW == 3)	//Test Button
		{
			RUN_TEST();
		}

		else 						//Calibration Combination (Frequency + Mode)
		{
			RUN_CAL();	
		}

	}

return 0;
}


/////////////////////////////////////////////////////////////////


int GET_BUTTON_VAL( void )
{
	int	ADCValue;

	set_ANALOG_BUTTON();
	//Perform Single Reading From ADC
	ADCON1bits.SAMP = 1; // start sampling then ...
	// after 2Tad go to conversion

	while (!ADCON1bits.DONE); // conversion done?
	ADCValue = ADCBUF0; // yes then get ADC value
	
	if (450 < ADCValue && ADCValue < 600)
	{
		return 1;	
	}

	else if (350 < ADCValue && ADCValue < 450)
	{
		return 2; 
	}

	else if (75 < ADCValue && ADCValue < 125)
	{
		return 3; 
	}

	else if (200 < ADCValue && ADCValue < 350)
	{
		return 4; 
	}

	else
	{
		return 0;
	}

}

/*****************************************************************************/
