/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        SPI DDS.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* For Use With AD9833 SPI DDS Signal Generator
* MCLK Signal For AD9833 Derived From PWM2.  1MHz Signal.
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include <dsp.h>
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"

/*****************************************************************************/
//DEFINES

	// Define 8 Bit Split (create 8bit word from 16bits)
	#define	LOBYTE(arg)	(arg & 0xFF)
	#define	HIBYTE(arg)	((char)(arg >> 8))

	#define	SPI_DDS_CS	LATDbits.LATD0

	#define	F10KHz		0x03	
	#define	F1KHz		0x02
	#define	F100Hz		0x01

	#define F10KHz_HI 	0X75C2
	#define	F10KHz_LO 	0X40A3

	#define F1KHz_HI 	0X5893
	#define	F1KHz_LO 	0X4010

	//Actualy 100.8063555Hz - For DSP Filter Operation
	#define F100Hz_HI 	0X69B4
	#define	F100Hz_LO 	0X4001

/*****************************************************************************/

void RESET_DDS_SPI()
{

	//Set Slow SPI - Falling Edge Trigger
	SPI1CONbits.SPRE = 6; 				//SECONDARY PRESCALE = 2:1
	SPI1CONbits.PPRE = 1; 				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 1;				//CLOCK ON ACTIVE->IDLE

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x01;						//SEND CONTROL BYTE UPPER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;			//SEND CONTROL BYTE LOWER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 1;  //Bring CS HIGH
	wait++;
	wait++;

	//Set Fast SPI - Rising Edge Trigger (For LCD Etc.)
	SPI1CONbits.SPRE = 7;				//SECONDARY PRESCALE = 1:1
	SPI1CONbits.PPRE = 1;				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 0;				//CLOCK ON IDLE->ACTIVE

}


////////////////////////////////////////


void RUN_DDS_SPI()
{

	//Set Slow SPI - Falling Edge Trigger
	SPI1CONbits.SPRE = 6; 				//SECONDARY PRESCALE = 2:1
	SPI1CONbits.PPRE = 1; 				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 1;				//CLOCK ON ACTIVE->IDLE

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 0; 				 	//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x00;						//SEND CONTROL BYTE UPPER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND CONTROL BYTE LOWER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 1;  					//Bring CS HIGH
	wait++;
	wait++;

	//Set Fast SPI - Rising Edge Trigger (For LCD Etc.)
	SPI1CONbits.SPRE = 7;				//SECONDARY PRESCALE = 1:1
	SPI1CONbits.PPRE = 1;				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 0;				//CLOCK ON IDLE->ACTIVE

}


////////////////////////////////////////


void SET_FREQ_DDS_SPI (int FREQ) //Set DDS Freqency
{

	int	HI_BYTE;
	int LO_BYTE;

	if (FREQ == F10KHz)
	{
	HI_BYTE = F10KHz_HI;
	LO_BYTE = F10KHz_LO;
	}
	else if (FREQ == F1KHz)
	{
	HI_BYTE = F1KHz_HI;
	LO_BYTE = F1KHz_LO;
	}
	else  //(FREQ == F100Hz)
	{
	HI_BYTE = F100Hz_HI;
	LO_BYTE = F100Hz_LO;
	}

	//Set Slow SPI - Falling Edge Trigger
	SPI1CONbits.SPRE = 6; 				//SECONDARY PRESCALE = 2:1
	SPI1CONbits.PPRE = 1; 				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 1;				//CLOCK ON ACTIVE->IDLE

	//Write Frequency Registers
	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x21;						//SEND CONTROL BYTE UPPER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND CONTROL BYTE LOWER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF= HIBYTE(HI_BYTE);			//SEND CONTROL BYTE UPPER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF= LOBYTE(HI_BYTE);			//SEND CONTROL BYTE LOWER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF= HIBYTE(LO_BYTE);			//SEND CONTROL BYTE UPPER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=LOBYTE(LO_BYTE);			//SEND CONTROL BYTE LOWER
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI_DDS_CS = 1;  					//Bring CS HIGH
	wait++;
	wait++;

	//Set Fast SPI - Rising Edge Trigger (For LCD Etc.)
	SPI1CONbits.SPRE = 7;				//SECONDARY PRESCALE = 1:1
	SPI1CONbits.PPRE = 1;				//PRIMARY PRESCALE = 16:1
	SPI1CONbits.CKE = 0;				//CLOCK ON IDLE->ACTIVE

}


/*****************************************************************************/
