/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        SPI GPIO.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* For Use With MSP23S08 SPI 8BIT GPIO - 10K Pulldown Required on SI Line
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include <dsp.h>
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"

/*****************************************************************************/
//DEFINES

	#define	SPI_LCD_CS		LATEbits.LATE5


/*****************************************************************************/


void SET_SPI_GPIO_OUTPUT_MODE()
{

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x40;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND ADDRESS BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND DATA BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 1;  					//Bring CS HIGH
	wait++;
	wait++;

}


////////////////////////////////////////


void SET_SPI_GPIO_INPUT_MODE()
{

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x40;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND ADDRESS BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0xFF;						//SEND DATA BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 1;  					//Bring CS HIGH
	wait++;
	wait++;

}


////////////////////////////////////////


void SET_SPI_GPIO_OUTPUT_VALUE(unsigned char OutputValue)
{

	int	wait;							//WAIT STATE
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x40;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x09;						//SEND ADDRESS BYTE
	while (SPI1STATbits.SPITBF == 1){}
	SPI1BUF=OutputValue;				//SEND DATA BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//ALLOW FRAME TO SEND
	SPI_LCD_CS = 1; 					//Bring CS HIGH
	wait++;
	wait++;

}


////////////////////////////////////////


unsigned char READ_SPI_GPIO_INPUT_VALUE()
{

	unsigned char READ_VAL;
	SET_SPI_GPIO_INPUT_MODE();
	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI_LCD_CS = 0;  					//Bring CS LOW
	wait++;
	wait++;
	SPI1BUF= 0x41;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x09;						//SEND ADDRESS BYTE
	wait = 0;
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR
	SPI1STATbits.SPIROV = 0; 			//CLEAR OVERUN FLAG
	READ_VAL = SPI1BUF;					//READ DATA BYTE
	SPI1STATbits.SPIROV = 0; 			//CLEAR OVERUN FLAG
	SPI1BUF=0x00;						//SEND EMPTY BYTE
	while (SPI1STATbits.SPIRBF == 0){}	//WAIT FOR READ BUFFER FULL
	READ_VAL = SPI1BUF;					//READ DATA BYTE
	SPI_LCD_CS = 1;  					//Bring CS HIGH
	wait++;
	wait++;

	SET_SPI_GPIO_OUTPUT_MODE();

return READ_VAL;
}


/*****************************************************************************/
