/**********************************************************************
*  2007 Miguel Rusch - MigDevelopments.
*
* FileName:        SPI MCP PGA.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30F2010
* Compiler:        MPLAB C30 v1.33.00 or higher
* IDE:             MPLAB IDE v7.20.01 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE IS PROVIDED AS IS.  MIGDEVELOPMENTS EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* MR              8/10/07   First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* For Use With MCP6S91 SPI PGA
*
**********************************************************************/
//INCLUDES

	#include <p30Fxxxx.h>
	#include "p30F4012.h"
	#include <dsp.h>
	#include "stdio.h"
	#include "math.h"
	#include "CustomDSP_Functions.h"

/*****************************************************************************/
//DEFINES

	#define		SPI_PGA_A	LATBbits.LATB5
	#define		SPI_PGA_V	LATDbits.LATD1

/*****************************************************************************/

void SET_SPI_PGA_GAIN(int GAIN, char AMP_CH)
{

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR

	if (AMP_CH == 'A'){
	SPI_PGA_A = 0;}  					//Bring CS LOW
	else {
	SPI_PGA_V = 0;}  					//Bring CS LOW

	wait++;
	wait++;
	SPI1BUF= 0x40;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR

	if (GAIN == 32){
	SPI1BUF = 0X07;}
	else if (GAIN == 16){
	SPI1BUF = 0X06;}
	else if (GAIN == 8){
	SPI1BUF = 0X04;}
	else if (GAIN == 4){
	SPI1BUF = 0X02;}
	else if (GAIN == 2){
	SPI1BUF = 0X01;}
	else {
	SPI1BUF = 0X00;} 					//Revert to GAIN = 1 in case of error.

	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR

	if (AMP_CH == 'A'){
	SPI_PGA_A = 1;}  					//Bring CS HIGH
	else {
	SPI_PGA_V = 1;}  					//Bring CS HIGH

	wait++;
	wait++;

}

////////////////////////////////////////

void SHUTDOWN_SPI_PGA(char AMP_CH)
{

	int	wait;
	wait = 0;
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR

	if (AMP_CH == 'A'){
	SPI_PGA_A = 0;}  					//Bring CS LOW
	else {
	SPI_PGA_V = 0;}  					//Bring CS LOW

	wait++;
	wait++;
	SPI1BUF= 0x20;						//SEND CONTROL BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	SPI1BUF=0x00;						//SEND DATA BYTE
	while (SPI1STATbits.SPITBF == 1){}	//WAIT FOR BUFFER CLEAR
	while	(wait <= 60){wait++;}		//WAIT FOR BUFFER CLEAR

	if (AMP_CH == 'A'){
	SPI_PGA_A = 1;}  					//Bring CS HIGH
	else {
	SPI_PGA_V = 1;}  					//Bring CS HIGH

	wait++;
	wait++;

}


/*****************************************************************************/
